@@ -15,6 +15,7 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
@@ -82,11 +83,11 @@ func init() {
 	})
 }
 
-// inputValsFromEnv populates GlobalFlags.Vars from the active environment. If we are working
+// inputValuesFromEnv populates GlobalFlags.Vars from the active environment. If we are working
 // inside a project, the "var" flag is not propagated to the schema definition. Instead, it
 // is used to evaluate the project file which can pass input values via the "values" block
 // to the schema.
-func inputValsFromEnv(cmd *cobra.Command, env *Env) error {
+func inputValuesFromEnv(cmd *cobra.Command, env *Env) error {
 	if fl := cmd.Flag(flagVar); fl == nil {
 		return nil
 	}
@@ -103,7 +104,7 @@ func inputValsFromEnv(cmd *cobra.Command, env *Env) error {
 	}
 	vars := strings.Join(pairs, ",")
 	if err := cmd.Flags().Set(flagVar, vars); err != nil {
-		return err
+		return fmt.Errorf("set flag %q: %w", flagVar, err)
 	}
 	return nil
 }
@@ -143,6 +144,20 @@ func (v Vars) String() string {
 	return "[" + b.String() + "]"
 }
 
+// Copy returns a copy of the current variables.
+func (v Vars) Copy() Vars {
+	vc := make(Vars)
+	for k := range v {
+		vc[k] = v[k]
+	}
+	return vc
+}
+
+// Replace overrides the variables.
+func (v *Vars) Replace(vc Vars) {
+	*v = vc
+}
+
 // Set implements pflag.Value.Set.
 func (v *Vars) Set(s string) error {
 	if *v == nil {
@@ -177,6 +192,7 @@ func (v *Vars) Type() string {
 
 const (
 	flagAllowDirty     = "allow-dirty"
+	flagEdit           = "edit"
 	flagAutoApprove    = "auto-approve"
 	flagBaseline       = "baseline"
 	flagConfig         = "config"
@@ -193,6 +209,7 @@ const (
 	flagGitBase        = "git-base"
 	flagGitDir         = "git-dir"
 	flagLatest         = "latest"
+	flagLockTimeout    = "lock-timeout"
 	flagLog            = "log"
 	flagRevisionSchema = "revisions-schema"
 	flagSchema         = "schema"
@@ -219,6 +236,10 @@ func addFlagDirFormat(set *pflag.FlagSet, target *string) {
 	set.StringVar(target, flagDirFormat, "atlas", "select migration file format")
 }
 
+func addFlagLockTimeout(set *pflag.FlagSet, target *time.Duration) {
+	set.DurationVar(target, flagLockTimeout, 10*time.Second, "set how long to wait for the database lock")
+}
+
 // addFlagURL adds a URL flag. If given, args[0] override the name, args[1] the shorthand, args[2] the default value.
 func addFlagDirURL(set *pflag.FlagSet, target *string, args ...string) {
 	name, short, val := flagDirURL, "", "file://migrations"
@@ -340,18 +361,33 @@ func maySetFlag(cmd *cobra.Command, name, envVal string) error {
 // were not set by the user and returns a callback to clear them
 // after it was set by the current environment.
 func resetFromEnv(cmd *cobra.Command) func() {
-	mayReset := make(map[string]string)
+	mayReset := make(map[string]func() error)
 	cmd.Flags().VisitAll(func(f *pflag.Flag) {
-		if !f.Changed {
-			mayReset[f.Name] = f.Value.String()
+		if f.Changed {
+			return
 		}
+		vs := f.Value.String()
+		r := func() error { return f.Value.Set(vs) }
+		if v, ok := f.Value.(*Vars); ok {
+			vs := v.Copy()
+			r = func() error {
+				v.Replace(vs)
+				return nil
+			}
+		} else if v, ok := f.Value.(pflag.SliceValue); ok {
+			vs := v.GetSlice()
+			r = func() error {
+				return v.Replace(vs)
+			}
+		}
+		mayReset[f.Name] = r
 	})
 	return func() {
-		for n, v := range mayReset {
-			if f := cmd.Flag(n); f != nil && f.Changed {
+		for name, reset := range mayReset {
+			if f := cmd.Flag(name); f != nil && f.Changed {
 				f.Changed = false
 				// Unexpected error, because this flag was set before.
-				cobra.CheckErr(f.Value.Set(v))
+				cobra.CheckErr(reset())
 			}
 		}
 	}
