@@ -14,12 +14,14 @@ import (
 	"io/fs"
 	"net/url"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"strconv"
 	"strings"
 	"text/template"
 	"time"
 
+	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	"ariga.io/atlas/cmd/atlas/internal/lint"
 	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/cmd/atlas/internal/migrate/ent/revision"
@@ -68,6 +70,7 @@ type migrateApplyFlags struct {
 	revisionSchema  string
 	dryRun          bool
 	logFormat       string
+	lockTimeout     time.Duration
 	allowDirty      bool   // allow working on a database that already has resources
 	fromVersion     string // compute pending files based on this version
 	baselineVersion string // apply with this version as baseline
@@ -97,7 +100,7 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 				if GlobalFlags.SelectedEnv == "" {
 					return migrateApplyRun(cmd, args, flags)
 				}
-				return migrateEnvsRun(migrateApplyRun, cmd, args, &flags)
+				return cmdEnvsRun(migrateApplyRun, setMigrateEnvFlags, cmd, args, &flags)
 			},
 		}
 	)
@@ -107,6 +110,7 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 	addFlagLog(cmd.Flags(), &flags.logFormat)
 	addFlagRevisionSchema(cmd.Flags(), &flags.revisionSchema)
 	addFlagDryRun(cmd.Flags(), &flags.dryRun)
+	addFlagLockTimeout(cmd.Flags(), &flags.lockTimeout)
 	cmd.Flags().StringVarP(&flags.fromVersion, flagFrom, "", "", "calculate pending files from the given version (including it)")
 	cmd.Flags().StringVarP(&flags.baselineVersion, flagBaseline, "", "", "start the first migration after the given baseline version")
 	cmd.Flags().StringVarP(&flags.txMode, flagTxMode, "", txModeFile, "set transaction mode [none, file, all]")
@@ -122,8 +126,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 		err   error
 	)
 	if len(args) > 0 {
-		count, err = strconv.Atoi(args[0])
-		if err != nil {
+		if count, err = strconv.Atoi(args[0]); err != nil {
 			return err
 		}
 		if count < 1 {
@@ -150,12 +153,12 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 	defer client.Close()
 	// Acquire a lock.
 	if l, ok := client.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(cmd.Context(), applyLockValue, 0)
+		unlock, err := l.Lock(cmd.Context(), applyLockValue, flags.lockTimeout)
 		if err != nil {
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
 		// If unlocking fails notify the user about it.
-		defer cobra.CheckErr(unlock())
+		defer func() { cobra.CheckErr(unlock()) }()
 	}
 	if err := checkRevisionSchemaClarity(cmd, client, flags.revisionSchema); err != nil {
 		return err
@@ -181,6 +184,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 	if v := flags.fromVersion; v != "" {
 		opts = append(opts, migrate.WithFromVersion(v))
 	}
+	report := cmdlog.NewMigrateApply(client, migrationDir)
 	ex, err := migrate.NewExecutor(client.Driver, migrationDir, rrw, opts...)
 	if err != nil {
 		return err
@@ -190,8 +194,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 		return err
 	}
 	if errors.Is(err, migrate.ErrNoPendingFiles) {
-		cmd.Println("No migration files to execute")
-		return nil
+		return reportApply(cmd, flags, report)
 	}
 	if l := len(pending); count == 0 || count >= l {
 		// Cannot apply more than len(pending) migration files.
@@ -202,7 +205,6 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 	if err != nil {
 		return err
 	}
-	report := cmdmigrate.NewApplyReport(client, migrationDir)
 	opts = append(opts, migrate.WithLogger(report))
 	if err := migrate.LogIntro(report, applied, pending); err != nil {
 		return err
@@ -242,27 +244,36 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags)
 			report.Log(migrate.LogDone{})
 		}
 	}
-	f := cmdmigrate.DefaultApplyTemplate
-	if v := flags.logFormat; v != "" {
-		f, err = template.New("format").Funcs(cmdmigrate.ApplyTemplateFuncs).Parse(v)
+	if err2 := reportApply(cmd, flags, report); err2 != nil {
 		if err != nil {
-			return fmt.Errorf("parse log format: %w", err)
-		}
-	}
-	if err2 := (&cmdmigrate.TemplateWriter{T: f, W: cmd.OutOrStdout()}).WriteReport(report); err2 != nil {
-		if err != nil {
-			return fmt.Errorf("%w: %v", err2, err)
+			err2 = fmt.Errorf("%w: %v", err, err2)
 		}
 		err = err2
 	}
 	return err
 }
 
+func reportApply(cmd *cobra.Command, flags migrateApplyFlags, r *cmdlog.MigrateApply) error {
+	var (
+		err error
+		f   = cmdlog.MigrateApplyTemplate
+	)
+	if v := flags.logFormat; v != "" {
+		f, err = template.New("format").Funcs(cmdlog.ApplyTemplateFuncs).Parse(v)
+		if err != nil {
+			return fmt.Errorf("parse log format: %w", err)
+		}
+	}
+	return f.Execute(cmd.OutOrStdout(), r)
+}
+
 type migrateDiffFlags struct {
+	edit              bool
 	desiredURLs       []string
 	dirURL, dirFormat string
 	devURL            string
 	schemas           []string
+	lockTimeout       time.Duration
 	revisionSchema    string // revision schema name
 	qualifier         string // optional table qualifier
 }
@@ -303,7 +314,9 @@ directory state to the desired schema. The desired state can be another connecte
 	addFlagDirFormat(cmd.Flags(), &flags.dirFormat)
 	addFlagRevisionSchema(cmd.Flags(), &flags.revisionSchema)
 	addFlagSchemas(cmd.Flags(), &flags.schemas)
+	addFlagLockTimeout(cmd.Flags(), &flags.lockTimeout)
 	cmd.Flags().StringVar(&flags.qualifier, flagQualifier, "", "qualify tables with custom qualifier when working on a single schema")
+	cmd.Flags().BoolVarP(&flags.edit, flagEdit, "", false, "edit the generated migration file(s)")
 	cobra.CheckErr(cmd.MarkFlagRequired(flagTo))
 	cobra.CheckErr(cmd.MarkFlagRequired(flagDevURL))
 	return cmd
@@ -318,12 +331,12 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 	defer dev.Close()
 	// Acquire a lock.
 	if l, ok := dev.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(cmd.Context(), "atlas_migrate_diff", 0)
+		unlock, err := l.Lock(cmd.Context(), "atlas_migrate_diff", flags.lockTimeout)
 		if err != nil {
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
 		// If unlocking fails notify the user about it.
-		defer cobra.CheckErr(unlock())
+		defer func() { cobra.CheckErr(unlock()) }()
 	}
 	// Open the migration directory.
 	u, err := url.Parse(flags.dirURL)
@@ -334,6 +347,9 @@ func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags) e
 	if err != nil {
 		return err
 	}
+	if flags.edit {
+		dir = &editDir{dir}
+	}
 	f, err := formatter(u)
 	if err != nil {
 		return err
@@ -636,7 +652,11 @@ func migrateLintRun(cmd *cobra.Command, _ []string, flags migrateLintFlags) erro
 	return err
 }
 
-type migrateNewFlags struct{ dirURL, dirFormat string }
+type migrateNewFlags struct {
+	edit      bool
+	dirURL    string
+	dirFormat string
+}
 
 // migrateNewCmd represents the 'atlas migrate new' subcommand.
 func migrateNewCmd() *cobra.Command {
@@ -665,6 +685,7 @@ func migrateNewCmd() *cobra.Command {
 	cmd.Flags().SortFlags = false
 	addFlagDirURL(cmd.Flags(), &flags.dirURL)
 	addFlagDirFormat(cmd.Flags(), &flags.dirFormat)
+	cmd.Flags().BoolVarP(&flags.edit, flagEdit, "", false, "edit the created migration file(s)")
 	return cmd
 }
 
@@ -677,6 +698,9 @@ func migrateNewRun(_ *cobra.Command, args []string, flags migrateNewFlags) error
 	if err != nil {
 		return err
 	}
+	if flags.edit {
+		dir = &editDir{dir}
+	}
 	f, err := formatter(u)
 	if err != nil {
 		return err
@@ -756,7 +780,7 @@ func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) err
 			return fmt.Errorf("acquiring database lock: %w", err)
 		}
 		// If unlocking fails notify the user about it.
-		defer cobra.CheckErr(unlock())
+		defer func() { cobra.CheckErr(unlock()) }()
 	}
 	if err := checkRevisionSchemaClarity(cmd, client, flags.revisionSchema); err != nil {
 		return err
@@ -895,6 +919,7 @@ func migrateStatusCmd() *cobra.Command {
 	)
 	cmd.Flags().SortFlags = false
 	addFlagURL(cmd.Flags(), &flags.url)
+	addFlagLog(cmd.Flags(), &flags.logFormat)
 	addFlagDirURL(cmd.Flags(), &flags.dirURL)
 	addFlagDirFormat(cmd.Flags(), &flags.dirFormat)
 	addFlagRevisionSchema(cmd.Flags(), &flags.revisionSchema)
@@ -914,19 +939,21 @@ func migrateStatusRun(cmd *cobra.Command, _ []string, flags migrateStatusFlags)
 	if err := checkRevisionSchemaClarity(cmd, client, flags.revisionSchema); err != nil {
 		return err
 	}
-	var format = cmdmigrate.DefaultStatusTemplate
+	report, err := (&cmdmigrate.StatusReporter{
+		Client: client,
+		Dir:    dir,
+		Schema: revisionSchemaName(client, flags.revisionSchema),
+	}).Report(cmd.Context())
+	if err != nil {
+		return err
+	}
+	format := cmdlog.MigrateStatusTemplate
 	if f := flags.logFormat; f != "" {
-		format, err = template.New("format").Funcs(cmdmigrate.StatusTemplateFuncs).Parse(f)
-		if err != nil {
+		if format, err = template.New("format").Funcs(cmdlog.StatusTemplateFuncs).Parse(f); err != nil {
 			return fmt.Errorf("parse log format: %w", err)
 		}
 	}
-	return (&cmdmigrate.StatusReporter{
-		Client:       client,
-		Dir:          dir,
-		ReportWriter: &cmdmigrate.TemplateWriter{T: format, W: cmd.OutOrStdout()},
-		Schema:       revisionSchemaName(client, flags.revisionSchema),
-	}).Report(cmd.Context())
+	return format.Execute(cmd.OutOrStdout(), report)
 }
 
 type migrateValidateFlags struct {
@@ -1347,8 +1374,8 @@ func selectScheme(urls []string) (string, error) {
 	if len(urls) == 0 {
 		return "", errors.New("at least one url is required")
 	}
-	for _, url := range urls {
-		parts := strings.SplitN(url, "://", 2)
+	for _, u := range urls {
+		parts := strings.SplitN(u, "://", 2)
 		switch current := parts[0]; {
 		case scheme == "":
 			scheme = current
@@ -1454,11 +1481,14 @@ func migrateFlagsFromEnv(cmd *cobra.Command) error {
 	if err != nil {
 		return err
 	}
-	return setFlagsFromEnv(cmd, activeEnv)
+	return setMigrateEnvFlags(cmd, activeEnv)
 }
 
-func setFlagsFromEnv(cmd *cobra.Command, env *Env) error {
-	if err := inputValsFromEnv(cmd, env); err != nil {
+func setMigrateEnvFlags(cmd *cobra.Command, env *Env) error {
+	if err := inputValuesFromEnv(cmd, env); err != nil {
+		return err
+	}
+	if err := maySetFlag(cmd, flagURL, env.URL); err != nil {
 		return err
 	}
 	if err := maySetFlag(cmd, flagDevURL, env.DevURL); err != nil {
@@ -1470,7 +1500,7 @@ func setFlagsFromEnv(cmd *cobra.Command, env *Env) error {
 	if err := maySetFlag(cmd, flagDirFormat, env.Migration.Format); err != nil {
 		return err
 	}
-	if err := maySetFlag(cmd, flagURL, env.URL); err != nil {
+	if err := maySetFlag(cmd, flagBaseline, env.Migration.Baseline); err != nil {
 		return err
 	}
 	if err := maySetFlag(cmd, flagRevisionSchema, env.Migration.RevisionsSchema); err != nil {
@@ -1481,6 +1511,13 @@ func setFlagsFromEnv(cmd *cobra.Command, env *Env) error {
 		if err := maySetFlag(cmd, flagLog, env.Log.Migrate.Apply); err != nil {
 			return err
 		}
+		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
+			return err
+		}
+	case "diff":
+		if err := maySetFlag(cmd, flagLockTimeout, env.Migration.LockTimeout); err != nil {
+			return err
+		}
 	case "lint":
 		if err := maySetFlag(cmd, flagLog, env.Log.Migrate.Lint); err != nil {
 			return err
@@ -1524,28 +1561,34 @@ func setFlagsFromEnv(cmd *cobra.Command, env *Env) error {
 	return nil
 }
 
-// migrateEnvsRun executes a given command on each of the configured environment.
-func migrateEnvsRun[F any](run func(*cobra.Command, []string, F) error, cmd *cobra.Command, args []string, flags *F) error {
+// cmdEnvsRun executes a given command on each of the configured environment.
+func cmdEnvsRun[F any](
+	run func(*cobra.Command, []string, F) error,
+	set func(*cobra.Command, *Env) error,
+	cmd *cobra.Command, args []string, flags *F,
+) error {
 	envs, err := LoadEnv(GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
 	if err != nil {
 		return err
 	}
 	var (
 		w     bytes.Buffer
+		out   = cmd.OutOrStdout()
 		reset = resetFromEnv(cmd)
 	)
-	cmd.SetOut(io.MultiWriter(cmd.OutOrStdout(), &w))
+	cmd.SetOut(io.MultiWriter(out, &w))
+	defer cmd.SetOut(out)
 	for i, e := range envs {
-		if err := setFlagsFromEnv(cmd, e); err != nil {
+		if err := set(cmd, e); err != nil {
 			return err
 		}
 		if err := run(cmd, args, *flags); err != nil {
 			return err
 		}
-		out := bytes.TrimLeft(w.Bytes(), " \t\r")
+		b := bytes.TrimLeft(w.Bytes(), " \t\r")
 		// In case a custom logging was configured, ensure there is
 		// a newline separator between the different environments.
-		if cmd.Flags().Changed(flagLog) && bytes.LastIndexByte(out, '\n') != len(out)-1 && i != len(envs)-1 {
+		if cmd.Flags().Changed(flagLog) && bytes.LastIndexByte(b, '\n') != len(b)-1 && i != len(envs)-1 {
 			cmd.Println()
 		}
 		reset()
@@ -1554,6 +1597,43 @@ func migrateEnvsRun[F any](run func(*cobra.Command, []string, F) error, cmd *cob
 	return nil
 }
 
+type editDir struct{ migrate.Dir }
+
+// WriteFile implements the migrate.Dir.WriteFile method.
+func (d *editDir) WriteFile(name string, b []byte) (err error) {
+	if name != migrate.HashFileName {
+		if b, err = edit(name, b); err != nil {
+			return err
+		}
+	}
+	return d.Dir.WriteFile(name, b)
+}
+
+// edit allows editing the file content using editor.
+func edit(name string, src []byte) ([]byte, error) {
+	path := filepath.Join(os.TempDir(), name)
+	if err := os.WriteFile(path, src, 0644); err != nil {
+		return nil, fmt.Errorf("write source content to temp file: %w", err)
+	}
+	defer os.Remove(path)
+	editor := "vi"
+	if e := os.Getenv("EDITOR"); e != "" {
+		editor = e
+	}
+	cmd := exec.Command("sh", "-c", editor+" "+path)
+	cmd.Stdin = os.Stdin
+	cmd.Stdout = os.Stdout
+	cmd.Stderr = os.Stderr
+	if err := cmd.Run(); err != nil {
+		return nil, fmt.Errorf("exec edit: %w", err)
+	}
+	b, err := os.ReadFile(path)
+	if err != nil {
+		return nil, fmt.Errorf("read edited temp file: %w", err)
+	}
+	return b, nil
+}
+
 type (
 	// dryRunDriver wraps a migrate.Driver without executing any SQL statements.
 	dryRunDriver struct{ migrate.Driver }
