@@ -6,7 +6,6 @@ package cmdapi
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/url"
 	"os"
@@ -16,6 +15,7 @@ import (
 	"ariga.io/atlas/sql/sqlclient"
 
 	"github.com/hashicorp/hcl/v2"
+	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/zclconf/go-cty/cty"
 )
@@ -77,6 +77,8 @@ type (
 	Migration struct {
 		Dir             string `spec:"dir"`
 		Format          string `spec:"format"`
+		Baseline        string `spec:"baseline"`
+		LockTimeout     string `spec:"lock_timeout"`
 		RevisionsSchema string `spec:"revisions_schema"`
 	}
 
@@ -105,6 +107,10 @@ type (
 			// Status configures the logging for 'migrate status'.
 			Status string `spec:"status"`
 		} `spec:"migrate"`
+		Schema struct {
+			// Apply configures the logging for 'schema apply'.
+			Apply string `spec:"apply"`
+		} `spec:"schema"`
 		schemahcl.DefaultExtension
 	}
 )
@@ -166,13 +172,18 @@ func (e *Env) Sources() ([]string, error) {
 	if !exists {
 		return nil, nil
 	}
-	if s, err := attr.String(); err == nil {
+	switch attr.V.Type() {
+	case cty.String:
+		s, err := attr.String()
+		if err != nil {
+			return nil, err
+		}
 		return []string{s}, nil
+	case cty.List(cty.String):
+		return attr.Strings()
+	default:
+		return nil, fmt.Errorf("expected src to be either a string or strings, got: %s", attr.V.Type().FriendlyName())
 	}
-	if s, err := attr.Strings(); err == nil {
-		return s, nil
-	}
-	return nil, errors.New("expected src to be either a string or a string array")
 }
 
 // asMap returns the extra attributes stored in the Env as a map[string]string.
@@ -186,9 +197,6 @@ func (e *Env) asMap() (map[string]string, error) {
 			m[attr.K] = v
 			continue
 		}
-		if lv, ok := attr.V.(*schemahcl.LiteralValue); ok {
-			m[attr.K] = lv.V
-		}
 		return nil, fmt.Errorf("expecting attr %q to be a literal, got: %T", attr.K, attr.V)
 	}
 	return m, nil
@@ -260,48 +268,54 @@ type sqlsrc struct {
 
 // exec executes the source block for getting the data.
 func (s *sqlsrc) exec() (cty.Value, error) {
-	attrs, diags := s.block.Body.JustAttributes()
+	var (
+		in struct {
+			URL    string   `hcl:"url"`
+			Query  string   `hcl:"query"`
+			Remain hcl.Body `hcl:",remain"`
+			Args   []any
+		}
+		values []cty.Value
+	)
+	if diags := gohcl.DecodeBody(s.block.Body, s.ctx, &in); diags.HasErrors() {
+		return cty.NilVal, s.errorf("decoding body: %v", diags)
+	}
+	attrs, diags := in.Remain.JustAttributes()
 	if diags.HasErrors() {
-		return cty.NilVal, diags
+		return cty.NilVal, s.errorf("getting attributes: %v", diags)
 	}
-	u, err := s.stringAttr(attrs, "url")
-	if err != nil {
-		return cty.NilVal, err
-	}
-	query, err := s.stringAttr(attrs, "query")
-	if err != nil {
-		return cty.NilVal, err
-	}
-	var args []any
 	if at, ok := attrs["args"]; ok {
 		switch v, diags := at.Expr.Value(s.ctx); {
 		case diags.HasErrors():
-			return cty.NilVal, s.errorf(`evaluating "args": %w`, err)
+			return cty.NilVal, s.errorf(`evaluating "args": %w`, diags)
 		case !v.CanIterateElements():
 			return cty.NilVal, s.errorf(`attribute "args" must be a list, got: %s`, v.Type())
 		default:
 			for it := v.ElementIterator(); it.Next(); {
 				switch _, v := it.Element(); v.Type() {
 				case cty.String:
-					args = append(args, v.AsString())
+					in.Args = append(in.Args, v.AsString())
 				case cty.Number:
 					f, _ := v.AsBigFloat().Float64()
-					args = append(args, f)
+					in.Args = append(in.Args, f)
 				case cty.Bool:
-					args = append(args, v.True())
+					in.Args = append(in.Args, v.True())
 				default:
-					return cty.NilVal, fmt.Errorf(`attribute "args" must contain primitive types, got: %s`, v.Type())
+					return cty.NilVal, s.errorf(`attribute "args" must be a list of strings, numbers or booleans, got: %s`, v.Type())
 				}
 			}
 		}
+		delete(attrs, "args")
 	}
-	var values []cty.Value
-	c, err := sqlclient.Open(context.Background(), u)
+	if len(attrs) > 0 {
+		return cty.NilVal, s.errorf("unexpected attributes: %v", attrs)
+	}
+	c, err := sqlclient.Open(context.Background(), in.URL)
 	if err != nil {
 		return cty.NilVal, s.errorf("opening connection: %w", err)
 	}
 	defer c.Close()
-	rows, err := c.QueryContext(context.Background(), query, args...)
+	rows, err := c.QueryContext(context.Background(), in.Query, in.Args...)
 	if err != nil {
 		return cty.NilVal, s.errorf("executing query: %w", err)
 	}
@@ -338,21 +352,6 @@ func (s *sqlsrc) exec() (cty.Value, error) {
 	return cty.ObjectVal(obj), nil
 }
 
-func (s *sqlsrc) stringAttr(attrs hcl.Attributes, name string) (string, error) {
-	at, ok := attrs[name]
-	if !ok {
-		return "", s.errorf("missing %q attribute", name)
-	}
-	u, diags := at.Expr.Value(s.ctx)
-	if diags.HasErrors() {
-		return "", s.errorf("evaluating %q attribute: %w", name, diags)
-	}
-	if u.Type() != cty.String {
-		return "", s.errorf("attribute %q must be a string, got: %s", name, u.Type())
-	}
-	return u.AsString(), nil
-}
-
 func (s *sqlsrc) errorf(format string, args ...any) error {
 	return fmt.Errorf("data.sql.%s: %w", s.block.Labels[1], fmt.Errorf(format, args...))
 }
