@@ -43,6 +43,7 @@ env "local" {
 	migration {
 		dir = "file://migrations"
 		format = atlas
+		lock_timeout = "1s"
 		revisions_schema = "revisions"
 	}
 	lint {
@@ -88,6 +89,7 @@ env "multi" {
 			Migration: &Migration{
 				Dir:             "file://migrations",
 				Format:          formatAtlas,
+				LockTimeout:     "1s",
 				RevisionsSchema: "revisions",
 			},
 			Lint: &Lint{
@@ -99,7 +101,7 @@ env "multi" {
 							{
 								Type: "destructive",
 								Attrs: []*schemahcl.Attr{
-									{K: "error", V: &schemahcl.LiteralValue{V: "true"}},
+									schemahcl.BoolAttr("error", true),
 								},
 							},
 						},
@@ -109,10 +111,10 @@ env "multi" {
 			DefaultExtension: schemahcl.DefaultExtension{
 				Extra: schemahcl.Resource{
 					Attrs: []*schemahcl.Attr{
-						{K: "bool", V: &schemahcl.LiteralValue{V: "true"}},
-						{K: "integer", V: &schemahcl.LiteralValue{V: "42"}},
-						{K: "src", V: &schemahcl.LiteralValue{V: `"./app.hcl"`}},
-						{K: "str", V: &schemahcl.LiteralValue{V: `"hello"`}},
+						schemahcl.BoolAttr("bool", true),
+						schemahcl.IntAttr("integer", 42),
+						schemahcl.StringAttr("src", "./app.hcl"),
+						schemahcl.StringAttr("str", "hello"),
 					},
 				},
 			},
