@@ -13,7 +13,9 @@ import (
 	"os"
 	"path/filepath"
 	"strings"
+	"text/template"
 
+	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
@@ -55,6 +57,7 @@ type schemaApplyFlags struct {
 	exclude     []string // List of glob patterns used to filter resources from applying (see schema.InspectOptions).
 	dryRun      bool     // Only show SQL on screen instead of applying it.
 	autoApprove bool     // Don't prompt for approval before applying SQL.
+	logFormat   string   // Log format.
 	dsn         string   // Deprecated: DSN is an alias for URL.
 }
 
@@ -86,22 +89,14 @@ migration.`,
   atlas schema apply -u "mariadb://user:pass@localhost:3306/dbname" --to file://schema.hcl
   atlas schema apply --url "postgres://user:pass@host:port/dbname?sslmode=disable" --to file://schema.hcl
   atlas schema apply -u "sqlite://file:ex1.db?_fk=1" --to file://schema.hcl`,
-			PreRunE: func(cmd *cobra.Command, args []string) error {
-				if err := schemaFlagsFromEnv(cmd, args); err != nil {
-					return err
-				}
-				if len(flags.paths) == 0 && len(flags.toURLs) == 0 {
-					return errors.New(`one of flag(s) "file" or "to" is required`)
-				}
-				// If the old -f flag is given convert them to the URL format. If both are given, cobra would throw an
-				// error since they are marked as mutually exclusive.
-				for _, p := range flags.paths {
-					flags.toURLs = append(flags.toURLs, "file://"+p)
-				}
-				return nil
+			PreRunE: func(cmd *cobra.Command, _ []string) error {
+				return dsn2url(cmd)
 			},
 			RunE: func(cmd *cobra.Command, args []string) error {
-				return schemaApplyRun(cmd, args, flags)
+				if GlobalFlags.SelectedEnv == "" {
+					return schemaApplyRun(cmd, args, flags)
+				}
+				return cmdEnvsRun(schemaApplyRun, setSchemaEnvFlags, cmd, args, &flags)
 			},
 		}
 	)
@@ -114,21 +109,42 @@ migration.`,
 	addFlagDevURL(cmd.Flags(), &flags.devURL)
 	addFlagDryRun(cmd.Flags(), &flags.dryRun)
 	addFlagAutoApprove(cmd.Flags(), &flags.autoApprove)
+	cmd.Flags().StringVarP(&flags.logFormat, flagLog, "", "", "custom logging using a Go template")
 	addFlagDSN(cmd.Flags(), &flags.dsn)
-	cobra.CheckErr(cmd.MarkFlagRequired(flagURL))
 	cmd.MarkFlagsMutuallyExclusive(flagFile, flagTo)
 	return cmd
 }
 
 func schemaApplyRun(cmd *cobra.Command, _ []string, flags schemaApplyFlags) error {
+	switch {
+	case flags.url == "":
+		return errors.New(`required flag(s) "url" not set`)
+	case len(flags.paths) == 0 && len(flags.toURLs) == 0:
+		return errors.New(`one of flag(s) "file" or "to" is required`)
+	case flags.logFormat != "" && !flags.dryRun && !flags.autoApprove:
+		return errors.New(`--log can only be used with --dry-run or --auto-approve`)
+	}
+	// If the old -f flag is given convert them to the URL format. If both are given,
+	// cobra would throw an error since they are marked as mutually exclusive.
+	for _, p := range flags.paths {
+		if !strings.Contains(p, "://") {
+			p = "file://" + p
+		}
+		flags.toURLs = append(flags.toURLs, p)
+	}
 	var (
-		ctx = cmd.Context()
-		err error
+		err    error
+		dev    *sqlclient.Client
+		ctx    = cmd.Context()
+		format = cmdlog.SchemaPlanTemplate
 	)
-	var dev *sqlclient.Client
+	if v := flags.logFormat; v != "" {
+		if format, err = template.New("format").Funcs(cmdlog.ApplyTemplateFuncs).Parse(v); err != nil {
+			return fmt.Errorf("parse log format: %w", err)
+		}
+	}
 	if flags.devURL != "" {
-		dev, err = sqlclient.Open(ctx, flags.devURL)
-		if err != nil {
+		if dev, err = sqlclient.Open(ctx, flags.devURL); err != nil {
 			return err
 		}
 		defer dev.Close()
@@ -162,19 +178,48 @@ func schemaApplyRun(cmd *cobra.Command, _ []string, flags schemaApplyFlags) erro
 	if err != nil {
 		return err
 	}
+	// Returning at this stage should
+	// not trigger the help message.
+	cmd.SilenceUsage = true
 	if len(changes) == 0 {
-		cmd.Println("Schema is synced, no changes to be made")
-		return nil
+		return format.Execute(cmd.OutOrStderr(), &cmdlog.SchemaApply{})
 	}
-	if err := summary(cmd, client, changes); err != nil {
-		return err
-	}
-	if !flags.dryRun && (flags.autoApprove || promptUser()) {
-		if err := client.ApplyChanges(ctx, changes); err != nil {
+	switch {
+	case flags.logFormat != "" && flags.autoApprove:
+		var (
+			applied int
+			plan    *migrate.Plan
+			cause   *cmdlog.StmtError
+		)
+		if plan, err = client.PlanChanges(cmd.Context(), "", changes); err != nil {
 			return err
 		}
+		if err = client.ApplyChanges(ctx, changes); err == nil {
+			applied = len(plan.Changes)
+		} else if i, ok := err.(interface{ Applied() int }); ok && i.Applied() < len(plan.Changes) {
+			applied, cause = i.Applied(), &cmdlog.StmtError{Stmt: plan.Changes[i.Applied()].Cmd, Text: err.Error()}
+		} else {
+			cause = &cmdlog.StmtError{Text: err.Error()}
+		}
+		apply := cmdlog.NewSchemaApply(cmdlog.NewEnv(client, nil), plan.Changes[:applied], plan.Changes[applied:], cause)
+		if err1 := format.Execute(cmd.OutOrStderr(), apply); err1 != nil {
+			if err != nil {
+				err1 = fmt.Errorf("%w: %v", err, err1)
+			}
+			err = err1
+		}
+		return err
+	default:
+		if err := summary(cmd, client, changes, format); err != nil {
+			return err
+		}
+		if !flags.dryRun && (flags.autoApprove || promptUser()) {
+			if err := client.ApplyChanges(ctx, changes); err != nil {
+				return err
+			}
+		}
+		return nil
 	}
-	return nil
 }
 
 type schemeCleanFlags struct {
@@ -193,7 +238,12 @@ func schemaCleanCmd() *cobra.Command {
 As a safety feature, 'atlas schema clean' will ask for confirmation before attempting to execute any SQL.`,
 			Example: `  atlas schema clean -u mysql://user:pass@localhost:3306/dbname
   atlas schema clean -u mysql://user:pass@localhost:3306/`,
-			PreRunE: schemaFlagsFromEnv,
+			PreRunE: func(cmd *cobra.Command, _ []string) error {
+				if err := schemaFlagsFromEnv(cmd); err != nil {
+					return err
+				}
+				return dsn2url(cmd)
+			},
 			RunE: func(cmd *cobra.Command, args []string) error {
 				return schemaCleanRun(cmd, args, flags)
 			},
@@ -221,7 +271,7 @@ func schemaCleanRun(cmd *cobra.Command, _ []string, flags schemeCleanFlags) erro
 		if err != nil {
 			return err
 		}
-		drop, err = c.RealmDiff(r, nil)
+		drop, err = c.RealmDiff(r, schema.NewRealm())
 		if err != nil {
 			return err
 		}
@@ -239,7 +289,7 @@ func schemaCleanRun(cmd *cobra.Command, _ []string, flags schemeCleanFlags) erro
 		cmd.Println("Nothing to drop")
 		return nil
 	}
-	if err := summary(cmd, c, drop); err != nil {
+	if err := summary(cmd, c, drop, cmdlog.SchemaPlanTemplate); err != nil {
 		return err
 	}
 	if flags.AutoApprove || promptUser() {
@@ -272,6 +322,12 @@ The database states can be read from a connected database, an HCL project or a m
 			Example: `  atlas schema diff --from mysql://user:pass@localhost:3306/test --to file://schema.hcl
   atlas schema diff --from mysql://user:pass@localhost:3306 --to file://schema_1.hcl --to file://schema_2.hcl
   atlas schema diff --from mysql://user:pass@localhost:3306 --to file://migrations`,
+			PreRunE: func(cmd *cobra.Command, _ []string) error {
+				if err := schemaFlagsFromEnv(cmd); err != nil {
+					return err
+				}
+				return dsn2url(cmd)
+			},
 			RunE: func(cmd *cobra.Command, args []string) error {
 				return schemaDiffRun(cmd, args, flags)
 			},
@@ -333,20 +389,7 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags) error
 	if err != nil {
 		return err
 	}
-	p, err := c.PlanChanges(ctx, "plan", diff)
-	if err != nil {
-		return err
-	}
-	if len(p.Changes) == 0 {
-		cmd.Println("Schemas are synced, no changes to be made.")
-	}
-	for _, c := range p.Changes {
-		if c.Comment != "" {
-			cmd.Println("--", strings.ToUpper(c.Comment[:1])+c.Comment[1:])
-		}
-		cmd.Println(c.Cmd)
-	}
-	return nil
+	return summary(cmd, c, diff, cmdlog.SchemaDiffTemplate)
 }
 
 type schemaInspectFlags struct {
@@ -380,7 +423,12 @@ flag.
   atlas schema inspect -u "mariadb://user:pass@localhost:3306/" --schema=schemaA,schemaB -s schemaC
   atlas schema inspect --url "postgres://user:pass@host:port/dbname?sslmode=disable"
   atlas schema inspect -u "sqlite://file:ex1.db?_fk=1"`,
-			PreRunE: schemaFlagsFromEnv,
+			PreRunE: func(cmd *cobra.Command, _ []string) error {
+				if err := schemaFlagsFromEnv(cmd); err != nil {
+					return err
+				}
+				return dsn2url(cmd)
+			},
 			RunE: func(cmd *cobra.Command, args []string) error {
 				return schemaInspectRun(cmd, args, flags)
 			},
@@ -482,40 +530,47 @@ func selectEnv(name string) (*Env, error) {
 	return envs[0], nil
 }
 
-func schemaFlagsFromEnv(cmd *cobra.Command, _ []string) error {
+func schemaFlagsFromEnv(cmd *cobra.Command) error {
 	activeEnv, err := selectEnv(GlobalFlags.SelectedEnv)
 	if err != nil {
 		return err
 	}
-	if err := inputValsFromEnv(cmd, activeEnv); err != nil {
+	return setSchemaEnvFlags(cmd, activeEnv)
+}
+
+func setSchemaEnvFlags(cmd *cobra.Command, env *Env) error {
+	if err := inputValuesFromEnv(cmd, env); err != nil {
 		return err
 	}
-	if err := dsn2url(cmd); err != nil {
+	if err := maySetFlag(cmd, flagURL, env.URL); err != nil {
 		return err
 	}
-	if err := maySetFlag(cmd, flagURL, activeEnv.URL); err != nil {
+	if err := maySetFlag(cmd, flagDevURL, env.DevURL); err != nil {
 		return err
 	}
-	if err := maySetFlag(cmd, flagDevURL, activeEnv.DevURL); err != nil {
-		return err
-	}
-	srcs, err := activeEnv.Sources()
+	srcs, err := env.Sources()
 	if err != nil {
 		return err
 	}
-	if err := maySetFlag(cmd, flagFile, strings.Join(srcs, "")); err != nil {
+	if err := maySetFlag(cmd, flagFile, strings.Join(srcs, ",")); err != nil {
 		return err
 	}
-	if s := strings.Join(activeEnv.Schemas, ","); s != "" {
+	if s := strings.Join(env.Schemas, ","); s != "" {
 		if err := maySetFlag(cmd, flagSchema, s); err != nil {
 			return err
 		}
 	}
-	if s := strings.Join(activeEnv.Exclude, ","); s != "" {
+	if s := strings.Join(env.Exclude, ","); s != "" {
 		if err := maySetFlag(cmd, flagExclude, s); err != nil {
 			return err
 		}
 	}
+	switch cmd.Name() {
+	case "apply":
+		if err := maySetFlag(cmd, flagLog, env.Log.Schema.Apply); err != nil {
+			return err
+		}
+	}
 	return nil
 }
 
@@ -552,19 +607,15 @@ func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *stateR
 	return diff, nil
 }
 
-func summary(cmd *cobra.Command, drv migrate.Driver, changes []schema.Change) error {
-	p, err := drv.PlanChanges(cmd.Context(), "", changes)
+func summary(cmd *cobra.Command, c *sqlclient.Client, changes []schema.Change, t *template.Template) error {
+	p, err := c.PlanChanges(cmd.Context(), "", changes)
 	if err != nil {
 		return err
 	}
-	cmd.Println("-- Planned Changes:")
-	for _, c := range p.Changes {
-		if c.Comment != "" {
-			cmd.Println("--", strings.ToUpper(c.Comment[:1])+c.Comment[1:])
-		}
-		cmd.Println(c.Cmd)
-	}
-	return nil
+	return t.Execute(
+		cmd.OutOrStdout(),
+		cmdlog.NewSchemaPlan(cmdlog.NewEnv(c, nil), p.Changes, nil),
+	)
 }
 
 const (
