@@ -14,6 +14,8 @@ import (
 	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlspec"
+
+	"github.com/zclconf/go-cty/cty"
 )
 
 // List of convert function types.
@@ -133,12 +135,20 @@ func Column(spec *sqlspec.Column, conv ConvertTypeFunc) (*schema.Column, error)
 			Null: spec.Null,
 		},
 	}
-	if spec.Default != nil {
-		switch d := spec.Default.(type) {
-		case *schemahcl.LiteralValue:
-			out.Default = &schema.Literal{V: d.V}
-		case *schemahcl.RawExpr:
-			out.Default = &schema.RawExpr{X: d.X}
+	if d := spec.Default; !d.IsNull() {
+		switch {
+		case d.Type() == cty.String:
+			out.Default = &schema.Literal{V: d.AsString()}
+		case d.Type() == cty.Number:
+			out.Default = &schema.Literal{V: d.AsBigFloat().String()}
+		case d.Type() == cty.Bool:
+			out.Default = &schema.Literal{V: strconv.FormatBool(d.True())}
+		case d.Type().IsCapsuleType():
+			x, ok := d.EncapsulatedValue().(*schemahcl.RawExpr)
+			if !ok {
+				return nil, fmt.Errorf("invalid default value %q", d.Type().FriendlyName())
+			}
+			out.Default = &schema.RawExpr{X: x.X}
 		default:
 			return nil, fmt.Errorf("unsupported value type for default: %T", d)
 		}
@@ -368,7 +378,7 @@ func FromColumn(col *schema.Column, columnTypeSpec ColumnTypeSpecFunc) (*sqlspec
 		},
 	}
 	if col.Default != nil {
-		lv, err := toValue(col.Default)
+		lv, err := ExprValue(col.Default)
 		if err != nil {
 			return nil, err
 		}
@@ -383,7 +393,7 @@ func FromGenExpr(x schema.GeneratedExpr, t func(string) string) *schemahcl.Resou
 	return &schemahcl.Resource{
 		Type: "as",
 		Attrs: []*schemahcl.Attr{
-			StrAttr("expr", x.Expr),
+			schemahcl.StringAttr("expr", x.Expr),
 			VarAttr("type", t(x.Type)),
 		},
 	}
@@ -421,39 +431,46 @@ func ConvertGenExpr(r *schemahcl.Resource, c *schema.Column, t func(string) stri
 	return nil
 }
 
-func toValue(expr schema.Expr) (schemahcl.Value, error) {
-	var (
-		v   string
-		err error
-	)
-	switch expr := expr.(type) {
+// ExprValue converts a schema.Expr to a cty.Value.
+func ExprValue(expr schema.Expr) (cty.Value, error) {
+	switch x := expr.(type) {
 	case *schema.RawExpr:
-		return &schemahcl.RawExpr{X: expr.X}, nil
+		return schemahcl.RawExprValue(&schemahcl.RawExpr{X: x.X}), nil
 	case *schema.Literal:
-		v, err = normalizeQuotes(expr.V)
-		if err != nil {
-			return nil, err
+		switch {
+		case oneOfPrefix(x.V, "0x", "0X", "0b", "0B", "b'", "B'", "x'", "X'"):
+			return schemahcl.RawExprValue(&schemahcl.RawExpr{X: x.V}), nil
+		case sqlx.IsQuoted(x.V, '\'', '"'):
+			// Normalize single quotes to double quotes.
+			s, err := sqlx.Unquote(x.V)
+			if err != nil {
+				return cty.NilVal, err
+			}
+			return cty.StringVal(s), nil
+		case x.V == "true", x.V == "false":
+			return cty.BoolVal(x.V == "true"), nil
+		case strings.Contains(x.V, "."):
+			f, err := strconv.ParseFloat(x.V, 64)
+			if err != nil {
+				return cty.NilVal, err
+			}
+			return cty.NumberFloatVal(f), nil
+		case sqlx.IsLiteralNumber(x.V):
+			i, err := strconv.ParseInt(x.V, 10, 64)
+			if err != nil {
+				return cty.NilVal, err
+			}
+			return cty.NumberIntVal(i), nil
+		default:
+			return cty.NilVal, fmt.Errorf("unsupported literal value %q", x.V)
 		}
-		return &schemahcl.LiteralValue{V: v}, nil
 	default:
-		return nil, fmt.Errorf("converting expr %T to literal value", expr)
+		return cty.NilVal, fmt.Errorf("converting expr %T to literal value", expr)
 	}
 }
 
-func normalizeQuotes(s string) (string, error) {
-	if len(s) < 2 {
-		return s, nil
-	}
-	// If string is quoted with single quotes:
-	if strings.HasPrefix(s, `'`) && strings.HasSuffix(s, `'`) {
-		uq := strings.ReplaceAll(s[1:len(s)-1], "''", "'")
-		return strconv.Quote(uq), nil
-	}
-	return s, nil
-}
-
 // FromIndex converts schema.Index to sqlspec.Index.
-func FromIndex(idx *schema.Index, partFns ...func(*schema.IndexPart, *sqlspec.IndexPart)) (*sqlspec.Index, error) {
+func FromIndex(idx *schema.Index, partFns ...func(*schema.Index, *schema.IndexPart, *sqlspec.IndexPart) error) (*sqlspec.Index, error) {
 	spec := &sqlspec.Index{Name: idx.Name, Unique: idx.Unique}
 	convertCommentFromSchema(idx.Attrs, &spec.Extra.Attrs)
 	spec.Parts = make([]*sqlspec.IndexPart, len(idx.Parts))
@@ -474,7 +491,9 @@ func FromIndex(idx *schema.Index, partFns ...func(*schema.IndexPart, *sqlspec.In
 			part.Expr = x.X
 		}
 		for _, f := range partFns {
-			f(p, part)
+			if err := f(idx, p, part); err != nil {
+				return nil, err
+			}
 		}
 		spec.Parts[i] = part
 	}
@@ -574,31 +593,39 @@ func externalRef(ref *schemahcl.Ref, sch *schema.Schema) (*schema.Table, *schema
 // is not in the provided schema.Schema other schemas in the connected schema.Realm
 // are searched as well.
 func findTable(ref *schemahcl.Ref, sch *schema.Schema) (*schema.Table, error) {
-	qualifier, tblName, err := tableName(ref)
+	qualifier, name, err := tableName(ref)
 	if err != nil {
 		return nil, err
 	}
-	// Search the same schema.
-	if qualifier == "" || qualifier == sch.Name {
-		tbl, ok := sch.Table(tblName)
-		if !ok {
-			return tbl, fmt.Errorf("sqlspec: table %q not found", tblName)
+	var (
+		matches []*schema.Table  // Found references.
+		schemas []*schema.Schema // Schemas to search.
+	)
+	switch {
+	case sch.Realm == nil || qualifier == sch.Name:
+		schemas = []*schema.Schema{sch}
+	case qualifier == "":
+		schemas = sch.Realm.Schemas
+	default:
+		s, ok := sch.Realm.Schema(qualifier)
+		if ok {
+			schemas = []*schema.Schema{s}
 		}
-		return tbl, nil
 	}
-	if sch.Realm == nil {
-		return nil, fmt.Errorf("sqlspec: table %s.%s not found", qualifier, tblName)
+	for _, s := range schemas {
+		t, ok := s.Table(name)
+		if ok {
+			matches = append(matches, t)
+		}
 	}
-	// Search for the table in another schemas in the realm.
-	sch, ok := sch.Realm.Schema(qualifier)
-	if !ok {
-		return nil, fmt.Errorf("sqlspec: schema %q not found", qualifier)
+	switch len(matches) {
+	case 1:
+		return matches[0], nil
+	case 0:
+		return nil, fmt.Errorf("sqlspec: table %q not found", name)
+	default:
+		return nil, fmt.Errorf("specutil: multiple tables found for %q", name)
 	}
-	tbl, ok := sch.Table(tblName)
-	if !ok {
-		return tbl, fmt.Errorf("sqlspec: table %q not found", tblName)
-	}
-	return tbl, nil
 }
 
 func tableName(ref *schemahcl.Ref) (qualifier, name string, err error) {
@@ -662,7 +689,7 @@ func convertCommentFromSpec(spec Attrer, attrs *[]schema.Attr) error {
 func convertCommentFromSchema(src []schema.Attr, trgt *[]*schemahcl.Attr) {
 	var c schema.Comment
 	if sqlx.Has(src, &c) {
-		*trgt = append(*trgt, StrAttr("comment", c.Text))
+		*trgt = append(*trgt, schemahcl.StringAttr("comment", c.Text))
 	}
 }
 
@@ -682,3 +709,12 @@ func Var(s string) string { return strings.ReplaceAll(s, " ", "_") }
 
 // FromVar is the inverse function of Var.
 func FromVar(s string) string { return strings.ReplaceAll(s, "_", " ") }
+
+func oneOfPrefix(s string, ps ...string) bool {
+	for _, p := range ps {
+		if strings.HasPrefix(s, p) {
+			return true
+		}
+	}
+	return false
+}
