@@ -10,13 +10,28 @@ import (
 	"reflect"
 	"strconv"
 	"strings"
+	"sync"
 
 	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/schema"
 )
 
+// DefaultDiff provides basic diffing capabilities for MySQL dialects.
+// Note, it is recommended to call Open, create a new Driver and use its
+// Differ when a database connection is available.
+var DefaultDiff schema.Differ = &sqlx.Diff{DiffDriver: &diff{}}
+
 // A diff provides a MySQL implementation for sqlx.DiffDriver.
-type diff struct{ conn }
+type diff struct {
+	conn
+	// charset to collation mapping.
+	// See, internal directory.
+	ch2co, co2ch struct {
+		sync.Once
+		v   map[string]string
+		err error
+	}
+}
 
 // SchemaAttrDiff returns a changeset for migrating schema attributes from one state to the other.
 func (d *diff) SchemaAttrDiff(from, to *schema.Schema) []schema.Change {
@@ -53,8 +68,8 @@ func (d *diff) TableAttrDiff(from, to *schema.Table) ([]schema.Change, error) {
 	if change := d.collationChange(from.Attrs, from.Schema.Attrs, to.Attrs); change != noChange {
 		changes = append(changes, change)
 	}
-	if _, ok := d.supportsCheck(); !ok && sqlx.Has(to.Attrs, &schema.Check{}) {
-		return nil, fmt.Errorf("version %q does not support CHECK constraints", d.version)
+	if !d.SupportsCheck() && sqlx.Has(to.Attrs, &schema.Check{}) {
+		return nil, fmt.Errorf("version %q does not support CHECK constraints", d.V)
 	}
 	// For MariaDB, we skip JSON CHECK constraints that were created by the databases,
 	// or by Atlas for older versions. These CHECK constraints (inlined on the columns)
@@ -79,7 +94,7 @@ func (d *diff) TableAttrDiff(from, to *schema.Table) ([]schema.Change, error) {
 }
 
 // ColumnChange returns the schema changes (if any) for migrating one column to the other.
-func (d *diff) ColumnChange(from, to *schema.Column) (schema.ChangeKind, error) {
+func (d *diff) ColumnChange(fromT *schema.Table, from, to *schema.Column) (schema.ChangeKind, error) {
 	change := sqlx.CommentChange(from.Attrs, to.Attrs)
 	if from.Type.Null != to.Type.Null {
 		change |= schema.ChangeNull
@@ -103,6 +118,18 @@ func (d *diff) ColumnChange(from, to *schema.Column) (schema.ChangeKind, error)
 	if changed {
 		change |= schema.ChangeGenerated
 	}
+	if changed, err = d.columnCharsetChanged(fromT, from, to); err != nil {
+		return schema.NoChange, err
+	}
+	if changed {
+		change |= schema.ChangeCharset
+	}
+	if changed, err = d.columnCollateChanged(fromT, from, to); err != nil {
+		return schema.NoChange, err
+	}
+	if changed {
+		change |= schema.ChangeCollate
+	}
 	return change, nil
 }
 
@@ -112,9 +139,9 @@ func (d *diff) IsGeneratedIndexName(_ *schema.Table, idx *schema.Index) bool {
 	// mysql-server/sql/sql_table.cc#add_functional_index_to_create_list
 	const f = "functional_index"
 	switch {
-	case d.supportsIndexExpr() && idx.Name == f:
+	case d.SupportsIndexExpr() && idx.Name == f:
 		return true
-	case d.supportsIndexExpr() && strings.HasPrefix(idx.Name+"_", f):
+	case d.SupportsIndexExpr() && strings.HasPrefix(idx.Name+"_", f):
 		i, err := strconv.ParseInt(strings.TrimLeft(idx.Name, idx.Name+"_"), 10, 64)
 		return err == nil && i > 1
 	case len(idx.Parts) == 0 || idx.Parts[0].C == nil:
@@ -140,9 +167,9 @@ func (*diff) IndexAttrChanged(from, to []schema.Attr) bool {
 }
 
 // IndexPartAttrChanged reports if the index-part attributes (collation or prefix) were changed.
-func (*diff) IndexPartAttrChanged(from, to *schema.IndexPart) bool {
+func (*diff) IndexPartAttrChanged(fromI, toI *schema.Index, i int) bool {
 	var s1, s2 SubPart
-	return sqlx.Has(from.Attrs, &s1) != sqlx.Has(to.Attrs, &s2) || s1.Len != s2.Len
+	return sqlx.Has(fromI.Parts[i].Attrs, &s1) != sqlx.Has(toI.Parts[i].Attrs, &s2) || s1.Len != s2.Len
 }
 
 // ReferenceChanged reports if the foreign key referential action was changed.
@@ -161,7 +188,7 @@ func (*diff) ReferenceChanged(from, to schema.ReferenceOption) bool {
 }
 
 // Normalize implements the sqlx.Normalizer interface.
-func (*diff) Normalize(from, to *schema.Table) {
+func (d *diff) Normalize(from, to *schema.Table) error {
 	indexes := make([]*schema.Index, 0, len(from.Indexes))
 	for _, idx := range from.Indexes {
 		// MySQL requires that foreign key columns be indexed; Therefore, if the child
@@ -175,6 +202,13 @@ func (*diff) Normalize(from, to *schema.Table) {
 		}
 	}
 	from.Indexes = indexes
+
+	// Avoid proposing changes to the table COLLATE or CHARSET
+	// in case only one of these properties is defined.
+	if err := d.defaultCollate(&to.Attrs); err != nil {
+		return err
+	}
+	return d.defaultCharset(&to.Attrs)
 }
 
 // collationChange returns the schema change for migrating the collation if
@@ -235,6 +269,46 @@ func (*diff) charsetChange(from, top, to []schema.Attr) schema.Change {
 	return noChange
 }
 
+// columnCharsetChange indicates if there is a change to the column charset.
+func (d *diff) columnCharsetChanged(fromT *schema.Table, from, to *schema.Column) (bool, error) {
+	if err := d.defaultCharset(&to.Attrs); err != nil {
+		return false, err
+	}
+	var (
+		fromC, topC, toC       schema.Charset
+		fromHas, topHas, toHas = sqlx.Has(from.Attrs, &fromC), sqlx.Has(fromT.Attrs, &topC), sqlx.Has(to.Attrs, &toC)
+	)
+	// Column was updated with custom CHARSET that was dropped.
+	// Hence, we should revert to the one defined on the table.
+	return fromHas && !toHas && topHas && fromC.V != topC.V ||
+		// Custom CHARSET was added to the column. Hence,
+		// Does not match the one defined in the table.
+		!fromHas && toHas && topHas && toC.V != topC.V ||
+		// CHARSET was explicitly changed.
+		fromHas && toHas && fromC.V != toC.V, nil
+
+}
+
+// columnCollateChanged indicates if there is a change to the column charset.
+func (d *diff) columnCollateChanged(fromT *schema.Table, from, to *schema.Column) (bool, error) {
+	if err := d.defaultCollate(&to.Attrs); err != nil {
+		return false, err
+	}
+	var (
+		fromC, topC, toC       schema.Collation
+		fromHas, topHas, toHas = sqlx.Has(from.Attrs, &fromC), sqlx.Has(fromT.Attrs, &topC), sqlx.Has(to.Attrs, &toC)
+	)
+	// Column was updated with custom COLLATE that was dropped.
+	// Hence, we should revert to the one defined on the table.
+	return fromHas && !toHas && topHas && fromC.V != topC.V ||
+		// Custom COLLATE was added to the column. Hence,
+		// Does not match the one defined in the table.
+		!fromHas && toHas && topHas && toC.V != topC.V ||
+		// COLLATE was explicitly changed.
+		fromHas && toHas && fromC.V != toC.V, nil
+
+}
+
 // autoIncChange returns the schema change for changing the AUTO_INCREMENT
 // attribute in case it is not the default.
 func (*diff) autoIncChange(from, to []schema.Attr) schema.Change {
@@ -289,8 +363,17 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 	}
 	var changed bool
 	switch fromT := fromT.(type) {
-	case *schema.BinaryType, *schema.BoolType, *schema.DecimalType, *schema.FloatType:
-		changed = mustFormat(fromT) != mustFormat(toT)
+	case *BitType, *schema.BinaryType, *schema.BoolType, *schema.DecimalType, *schema.FloatType,
+		*schema.JSONType, *schema.StringType, *schema.SpatialType, *schema.TimeType:
+		ft, err := FormatType(fromT)
+		if err != nil {
+			return false, err
+		}
+		tt, err := FormatType(toT)
+		if err != nil {
+			return false, err
+		}
+		changed = ft != tt
 	case *schema.EnumType:
 		toT := toT.(*schema.EnumType)
 		changed = !sqlx.ValuesEqual(fromT.Values, toT.Values)
@@ -298,7 +381,7 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 		toT := toT.(*schema.IntegerType)
 		// MySQL v8.0.19 dropped both display-width
 		// and zerofill from the information schema.
-		if d.supportsDisplayWidth() {
+		if d.SupportsDisplayWidth() {
 			ft, _, _, err := parseColumn(fromT.T)
 			if err != nil {
 				return false, err
@@ -310,20 +393,6 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 			fromT.T, toT.T = ft[0], tt[0]
 		}
 		changed = fromT.T != toT.T || fromT.Unsigned != toT.Unsigned
-	case *schema.JSONType:
-		toT := toT.(*schema.JSONType)
-		changed = fromT.T != toT.T
-	case *schema.StringType:
-		changed = mustFormat(fromT) != mustFormat(toT)
-	case *schema.SpatialType:
-		toT := toT.(*schema.SpatialType)
-		changed = fromT.T != toT.T
-	case *schema.TimeType:
-		toT := toT.(*schema.TimeType)
-		changed = fromT.T != toT.T
-	case *BitType:
-		toT := toT.(*BitType)
-		changed = fromT.T != toT.T
 	case *SetType:
 		toT := toT.(*SetType)
 		changed = !sqlx.ValuesEqual(fromT.Values, toT.Values)
@@ -487,3 +556,45 @@ search:
 	}
 	return false
 }
+
+// defaultCollate appends the default COLLATE to the attributes in case a
+// custom character-set was defined for the element and the COLLATE was not.
+func (d *diff) defaultCollate(attrs *[]schema.Attr) error {
+	var charset schema.Charset
+	if !sqlx.Has(*attrs, &charset) || sqlx.Has(*attrs, &schema.Collation{}) {
+		return nil
+	}
+	d.ch2co.Do(func() {
+		d.ch2co.v, d.ch2co.err = d.CharsetToCollate()
+	})
+	if d.ch2co.err != nil {
+		return d.ch2co.err
+	}
+	v, ok := d.ch2co.v[charset.V]
+	if !ok {
+		return fmt.Errorf("mysql: unknown character set: %q", charset.V)
+	}
+	schema.ReplaceOrAppend(attrs, &schema.Collation{V: v})
+	return nil
+}
+
+// defaultCharset appends the default CHARSET to the attributes in case a
+// custom collation was defined for the element and the CHARSET was not.
+func (d *diff) defaultCharset(attrs *[]schema.Attr) error {
+	var collate schema.Collation
+	if !sqlx.Has(*attrs, &collate) || sqlx.Has(*attrs, &schema.Charset{}) {
+		return nil
+	}
+	d.co2ch.Do(func() {
+		d.co2ch.v, d.co2ch.err = d.CollateToCharset()
+	})
+	if d.co2ch.err != nil {
+		return d.co2ch.err
+	}
+	v, ok := d.co2ch.v[collate.V]
+	if !ok {
+		return fmt.Errorf("mysql: unknown collation: %q", collate.V)
+	}
+	schema.ReplaceOrAppend(attrs, &schema.Charset{V: v})
+	return nil
+}
