@@ -62,11 +62,11 @@ func TestDiff_TableDiff(t *testing.T) {
 		},
 		{
 			name: "add collation",
-			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
-			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Collation{V: "latin1"}}},
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Charset{V: "latin1"}}},
+			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Collation{V: "latin1_bin"}}},
 			wantChanges: []schema.Change{
 				&schema.AddAttr{
-					A: &schema.Collation{V: "latin1"},
+					A: &schema.Collation{V: "latin1_bin"},
 				},
 			},
 		},
@@ -83,22 +83,12 @@ func TestDiff_TableDiff(t *testing.T) {
 		},
 		{
 			name: "modify collation",
-			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Collation{V: "utf8"}}},
-			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Collation{V: "latin1"}}},
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Charset{V: "latin1"}, &schema.Collation{V: "latin1_swedish_ci"}}},
+			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Collation{V: "latin1_bin"}}},
 			wantChanges: []schema.Change{
 				&schema.ModifyAttr{
-					From: &schema.Collation{V: "utf8"},
-					To:   &schema.Collation{V: "latin1"},
-				},
-			},
-		},
-		{
-			name: "add charset",
-			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}},
-			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Charset{V: "hebrew"}}},
-			wantChanges: []schema.Change{
-				&schema.AddAttr{
-					A: &schema.Charset{V: "hebrew"},
+					From: &schema.Collation{V: "latin1_swedish_ci"},
+					To:   &schema.Collation{V: "latin1_bin"},
 				},
 			},
 		},
@@ -115,12 +105,16 @@ func TestDiff_TableDiff(t *testing.T) {
 		},
 		{
 			name: "modify charset",
-			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Charset{V: "hebrew"}}},
-			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Charset{V: "binary"}}},
+			from: &schema.Table{Name: "users", Schema: &schema.Schema{Name: "public"}, Attrs: []schema.Attr{&schema.Charset{V: "utf8"}, &schema.Collation{V: "utf8_general_ci"}}},
+			to:   &schema.Table{Name: "users", Attrs: []schema.Attr{&schema.Charset{V: "utf8mb4"}}},
 			wantChanges: []schema.Change{
 				&schema.ModifyAttr{
-					From: &schema.Charset{V: "hebrew"},
-					To:   &schema.Charset{V: "binary"},
+					From: &schema.Charset{V: "utf8"},
+					To:   &schema.Charset{V: "utf8mb4"},
+				},
+				&schema.ModifyAttr{
+					From: &schema.Collation{V: "utf8_general_ci"},
+					To:   &schema.Collation{V: "utf8mb4_0900_ai_ci"},
 				},
 			},
 		},
@@ -229,6 +223,110 @@ func TestDiff_TableDiff(t *testing.T) {
 				},
 			}
 		}(),
+		// Custom CHARSET was dropped.
+		func() testcase {
+			var (
+				from = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					SetCollation("utf8_general_ci").
+					AddColumns(schema.NewStringColumn("c1", "text").SetCharset("latin1"))
+				to = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					AddColumns(schema.NewStringColumn("c1", "text"))
+			)
+			return testcase{
+				name: "columns",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.ModifyColumn{
+						From:   from.Columns[0],
+						To:     to.Columns[0],
+						Change: schema.ChangeCharset,
+					},
+				},
+			}
+		}(),
+		// Custom CHARSET was added.
+		func() testcase {
+			var (
+				from = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					SetCollation("utf8_general_ci").
+					AddColumns(schema.NewStringColumn("c1", "text"))
+				to = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					AddColumns(schema.NewStringColumn("c1", "text").SetCharset("latin1"))
+			)
+			return testcase{
+				name: "columns",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.ModifyColumn{
+						From:   from.Columns[0],
+						To:     to.Columns[0],
+						Change: schema.ChangeCharset | schema.ChangeCollate,
+					},
+				},
+			}
+		}(),
+		// Custom CHARSET was changed.
+		func() testcase {
+			var (
+				from = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					SetCollation("utf8_general_ci").
+					AddColumns(schema.NewStringColumn("c1", "text").SetCharset("hebrew"))
+				to = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					SetCollation("utf8_general_ci").
+					AddColumns(schema.NewStringColumn("c1", "text").SetCharset("latin1"))
+			)
+			return testcase{
+				name: "columns",
+				from: from,
+				to:   to,
+				wantChanges: []schema.Change{
+					&schema.ModifyColumn{
+						From:   from.Columns[0],
+						To:     to.Columns[0],
+						Change: schema.ChangeCharset | schema.ChangeCollate,
+					},
+				},
+			}
+		}(),
+		// Nop CHARSET change.
+		func() testcase {
+			var (
+				from = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCharset("utf8").
+					SetCollation("utf8_general_ci").
+					AddColumns(
+						schema.NewStringColumn("c1", "text").SetCharset("utf8"),
+						schema.NewStringColumn("c2", "text"),
+					)
+				to = schema.NewTable("t1").
+					SetSchema(schema.New("public")).
+					SetCollation("utf8_general_ci").
+					AddColumns(
+						schema.NewStringColumn("c1", "text"),
+						schema.NewStringColumn("c2", "text").SetCharset("utf8"),
+					)
+			)
+			return testcase{
+				name: "columns",
+				from: from,
+				to:   to,
+			}
+		}(),
 		func() testcase {
 			var (
 				s    = schema.New("public")
@@ -525,3 +623,16 @@ func TestDiff_RealmDiff(t *testing.T) {
 		&schema.AddTable{T: to.Schemas[1].Tables[0]},
 	}, changes)
 }
+
+func TestDefaultDiff(t *testing.T) {
+	changes, err := DefaultDiff.SchemaDiff(
+		schema.New("public").
+			AddTables(
+				schema.NewTable("users").AddColumns(schema.NewIntColumn("id", "int")),
+			),
+		schema.New("public"),
+	)
+	require.NoError(t, err)
+	require.Len(t, changes, 1)
+	require.IsType(t, &schema.DropTable{}, changes[0])
+}
