@@ -17,6 +17,11 @@ import (
 	"ariga.io/atlas/sql/schema"
 )
 
+// DefaultDiff provides basic diffing capabilities for PostgreSQL dialects.
+// Note, it is recommended to call Open, create a new Driver and use its Differ
+// when a database connection is available.
+var DefaultDiff schema.Differ = &sqlx.Diff{DiffDriver: &diff{}}
+
 // A diff provides a PostgreSQL implementation for sqlx.DiffDriver.
 type diff struct{ conn }
 
@@ -78,15 +83,19 @@ func (d *diff) defaultChanged(from, to *schema.Column) (bool, error) {
 	if ok1 != ok2 {
 		return true, nil
 	}
-	if trimCast(d1) == trimCast(d2) {
+	if trimCast(d1) == trimCast(d2) || quote(d1) == quote(d2) {
 		return false, nil
 	}
-	// Use database comparison in case of mismatch (e.g. `SELECT ARRAY[1] = '{1}'::int[]`).
-	equals, err := d.valuesEqual(d1, d2)
-	if err != nil {
-		return false, err
+	var (
+		err    error
+		equals bool
+	)
+	// In case a database connection is available (not the DefaultDiff), we use the
+	// database comparison in case of mismatch (e.g. `SELECT ARRAY[1] = '{1}'::int[]`).
+	if d.conn.ExecQuerier != nil {
+		equals, err = d.valuesEqual(d1, d2)
 	}
-	return !equals, nil
+	return !equals, err
 }
 
 // generatedChanged reports if the generated expression of a column was changed.
@@ -164,18 +173,39 @@ func (*diff) IndexAttrChanged(from, to []schema.Attr) bool {
 	if sqlx.Has(from, &p1) != sqlx.Has(to, &p2) || (p1.P != p2.P && p1.P != sqlx.MayWrap(p2.P)) {
 		return true
 	}
+	if indexIncludeChanged(from, to) {
+		return true
+	}
 	s1, ok1 := indexStorageParams(from)
 	s2, ok2 := indexStorageParams(to)
 	return ok1 != ok2 || ok1 && *s1 != *s2
 }
 
 // IndexPartAttrChanged reports if the index-part attributes were changed.
-func (*diff) IndexPartAttrChanged(from, to *schema.IndexPart) bool {
+func (*diff) IndexPartAttrChanged(fromI, toI *schema.Index, i int) bool {
+	from, to := fromI.Parts[i], toI.Parts[i]
 	p1 := &IndexColumnProperty{NullsFirst: from.Desc, NullsLast: !from.Desc}
 	sqlx.Has(from.Attrs, p1)
 	p2 := &IndexColumnProperty{NullsFirst: to.Desc, NullsLast: !to.Desc}
 	sqlx.Has(to.Attrs, p2)
-	return p1.NullsFirst != p2.NullsFirst || p1.NullsLast != p2.NullsLast
+	if p1.NullsFirst != p2.NullsFirst || p1.NullsLast != p2.NullsLast {
+		return true
+	}
+	var fromOp, toOp IndexOpClass
+	switch fromHas, toHas := sqlx.Has(from.Attrs, &fromOp), sqlx.Has(to.Attrs, &toOp); {
+	case fromHas && toHas:
+		return !fromOp.Equal(&toOp)
+	case toHas:
+		// Report a change if a non-default operator class was added.
+		d, err := toOp.DefaultFor(toI, toI.Parts[i])
+		return !d && err == nil
+	case fromHas:
+		// Report a change if a non-default operator class was removed.
+		d, err := fromOp.DefaultFor(fromI, fromI.Parts[i])
+		return !d && err == nil
+	default:
+		return false
+	}
 }
 
 // ReferenceChanged reports if the foreign key referential action was changed.
@@ -203,7 +233,7 @@ func (d *diff) typeChanged(from, to *schema.Column) (bool, error) {
 	switch fromT := fromT.(type) {
 	case *schema.BinaryType, *BitType, *schema.BoolType, *schema.DecimalType, *schema.FloatType,
 		*IntervalType, *schema.IntegerType, *schema.JSONType, *SerialType, *schema.SpatialType,
-		*schema.StringType, *schema.TimeType, *NetworkType, *UserDefinedType:
+		*schema.StringType, *schema.TimeType, *TextSearchType, *NetworkType, *UserDefinedType:
 		t1, err := FormatType(toT)
 		if err != nil {
 			return false, err
@@ -352,6 +382,20 @@ func indexStorageParams(attrs []schema.Attr) (*IndexStorageParams, bool) {
 	return s, true
 }
 
+// indexIncludeChanged reports if the INCLUDE attribute clause was changed.
+func indexIncludeChanged(from, to []schema.Attr) bool {
+	var fromI, toI IndexInclude
+	if sqlx.Has(from, &fromI) != sqlx.Has(to, &toI) || len(fromI.Columns) != len(toI.Columns) {
+		return true
+	}
+	for i := range fromI.Columns {
+		if fromI.Columns[i].Name != toI.Columns[i].Name {
+			return true
+		}
+	}
+	return false
+}
+
 func trimCast(s string) string {
 	i := strings.LastIndex(s, "::")
 	if i == -1 {
