@@ -5,14 +5,18 @@
 package postgres
 
 import (
+	"bytes"
 	"context"
 	"database/sql"
+	"encoding/json"
 	"fmt"
 	"regexp"
 	"strconv"
 	"strings"
+	"sync"
 
 	"ariga.io/atlas/sql/internal/sqlx"
+	"ariga.io/atlas/sql/postgres/internal/postgresop"
 	"ariga.io/atlas/sql/schema"
 )
 
@@ -318,12 +322,15 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 	names := make(map[string]*schema.Index)
 	for rows.Next() {
 		var (
-			uniq, primary, included                       bool
-			table, name, typ                              string
-			desc, nullsfirst, nullslast                   sql.NullBool
-			column, contype, pred, expr, comment, options sql.NullString
+			uniq, primary, included                                               bool
+			table, name, typ                                                      string
+			desc, nullsfirst, nullslast, opcdefault                               sql.NullBool
+			column, constraints, pred, expr, comment, options, opcname, opcparams sql.NullString
 		)
-		if err := rows.Scan(&table, &name, &typ, &column, &included, &primary, &uniq, &contype, &pred, &expr, &desc, &nullsfirst, &nullslast, &comment, &options); err != nil {
+		if err := rows.Scan(
+			&table, &name, &typ, &column, &included, &primary, &uniq, &constraints, &pred, &expr,
+			&desc, &nullsfirst, &nullslast, &comment, &options, &opcname, &opcdefault, &opcparams,
+		); err != nil {
 			return fmt.Errorf("postgres: scanning indexes for schema %q: %w", s.Name, err)
 		}
 		t, ok := s.Table(table)
@@ -343,8 +350,14 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 			if sqlx.ValidString(comment) {
 				idx.Attrs = append(idx.Attrs, &schema.Comment{Text: comment.String})
 			}
-			if sqlx.ValidString(contype) {
-				idx.Attrs = append(idx.Attrs, &ConType{T: contype.String})
+			if sqlx.ValidString(constraints) {
+				var m map[string]string
+				if err := json.Unmarshal([]byte(constraints.String), &m); err != nil {
+					return fmt.Errorf("postgres: unmarshaling index constraints: %w", err)
+				}
+				for n, t := range m {
+					idx.Attrs = append(idx.Attrs, &Constraint{N: n, T: t})
+				}
 			}
 			if sqlx.ValidString(pred) {
 				idx.Attrs = append(idx.Attrs, &IndexPredicate{P: pred.String})
@@ -395,10 +408,26 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 		default:
 			return fmt.Errorf("postgres: invalid part for index %q", idx.Name)
 		}
+		if err := mayAppendOps(part, opcname.String, opcparams.String, opcdefault.Bool); err != nil {
+			return err
+		}
 	}
 	return nil
 }
 
+// mayAppendOps appends an operator_class attribute to the part in case it is not the default.
+func mayAppendOps(part *schema.IndexPart, name string, params string, defaults bool) error {
+	if name == "" || defaults && params == "" {
+		return nil
+	}
+	op := &IndexOpClass{Name: name, Default: defaults}
+	if err := op.parseParams(params); err != nil {
+		return err
+	}
+	part.Attrs = append(part.Attrs, op)
+	return nil
+}
+
 // partitions builds the partition each table in the schema.
 func (i *inspect) partitions(s *schema.Schema) error {
 	for _, t := range s.Tables {
@@ -694,6 +723,13 @@ type (
 		SequenceName string
 	}
 
+	// A TextSearchType defines full text search types.
+	// https://www.postgresql.org/docs/current/datatype-textsearch.html
+	TextSearchType struct {
+		schema.Type
+		T string
+	}
+
 	// A UUIDType defines a UUID type.
 	UUIDType struct {
 		schema.Type
@@ -706,10 +742,11 @@ type (
 		T string
 	}
 
-	// ConType describes constraint type.
+	// Constraint describes a postgres constraint.
 	// https://postgresql.org/docs/current/catalog-pg-constraint.html
-	ConType struct {
+	Constraint struct {
 		schema.Attr
+		N string // constraint name
 		T string // c, f, p, u, t, x.
 	}
 
@@ -772,8 +809,17 @@ type (
 		Columns []*schema.Column
 	}
 
-	// Concurrently describes the CONCURRENTLY clause to instruct Postgres
-	// to build or drop the index concurrently without blocking the current table.
+	// IndexOpClass describers operator class of the index part.
+	// https://www.postgresql.org/docs/current/indexes-opclass.html.
+	IndexOpClass struct {
+		schema.Attr
+		Name    string                  // Name of the operator class.
+		Default bool                    // If it is the default operator class.
+		Params  []struct{ N, V string } // Optional parameters.
+	}
+
+	// Concurrently describes the CONCURRENTLY clause to instruct Postgres to
+	// build or drop the index concurrently without blocking the current table.
 	// https://www.postgresql.org/docs/current/sql-createindex.html#SQL-CREATEINDEX-CONCURRENTLY
 	Concurrently struct {
 		schema.Attr
@@ -817,7 +863,7 @@ type (
 )
 
 // IsUnique reports if the type is unique constraint.
-func (c ConType) IsUnique() bool { return strings.ToLower(c.T) == "u" }
+func (c Constraint) IsUnique() bool { return strings.ToLower(c.T) == "u" }
 
 // IntegerType returns the underlying integer type this serial type represents.
 func (s *SerialType) IntegerType() *schema.IntegerType {
@@ -852,6 +898,112 @@ func (s *SerialType) sequence(t *schema.Table, c *schema.Column) string {
 	return fmt.Sprintf("%s_%s_seq", t.Name, c.Name)
 }
 
+var (
+	opsOnce    sync.Once
+	defaultOps map[postgresop.Class]bool
+)
+
+// DefaultFor reports if the operator_class is the default for the index part.
+func (o *IndexOpClass) DefaultFor(idx *schema.Index, part *schema.IndexPart) (bool, error) {
+	// Explicitly defined as the default (Usually, it comes from the inspection).
+	if o.Default && len(o.Params) == 0 {
+		return true, nil
+	}
+	it := &IndexType{T: IndexTypeBTree}
+	if sqlx.Has(idx.Attrs, it) {
+		it.T = strings.ToUpper(it.T)
+	}
+	// The key type must be known to check if it is the default op_class.
+	if part.X != nil || len(o.Params) > 0 {
+		return false, nil
+	}
+	opsOnce.Do(func() {
+		defaultOps = make(map[postgresop.Class]bool, len(postgresop.Classes))
+		for _, op := range postgresop.Classes {
+			if op.Default {
+				defaultOps[postgresop.Class{Name: op.Name, Method: op.Method, Type: op.Type}] = true
+			}
+		}
+	})
+	var (
+		t   string
+		err error
+	)
+	switch typ := part.C.Type.Type.(type) {
+	case *schema.EnumType:
+		t = "anyenum"
+	case *ArrayType:
+		t = "anyarray"
+	default:
+		t, err = FormatType(typ)
+		if err != nil {
+			return false, fmt.Errorf("postgres: format operator-class type %T: %w", typ, err)
+		}
+	}
+	return defaultOps[postgresop.Class{Name: o.Name, Method: it.T, Type: t}], nil
+}
+
+// Equal reports whether o and x are the same operator class.
+func (o *IndexOpClass) Equal(x *IndexOpClass) bool {
+	if o.Name != x.Name || o.Default != x.Default || len(o.Params) != len(x.Params) {
+		return false
+	}
+	for i := range o.Params {
+		if o.Params[i].N != x.Params[i].N || o.Params[i].V != x.Params[i].V {
+			return false
+		}
+	}
+	return true
+}
+
+// String returns the string representation of the operator class.
+func (o *IndexOpClass) String() string {
+	if len(o.Params) == 0 {
+		return o.Name
+	}
+	var b strings.Builder
+	b.WriteString(o.Name)
+	b.WriteString("(")
+	for i, p := range o.Params {
+		if i > 0 {
+			b.WriteString(", ")
+		}
+		b.WriteString(p.N)
+		b.WriteString("=")
+		b.WriteString(p.V)
+	}
+	b.WriteString(")")
+	return b.String()
+}
+
+// UnmarshalText parses the operator class from its string representation.
+func (o *IndexOpClass) UnmarshalText(text []byte) error {
+	i := bytes.IndexByte(text, '(')
+	if i == -1 {
+		o.Name = string(text)
+		return nil
+	}
+	o.Name = string(text[:i])
+	return o.parseParams(string(text[i:]))
+}
+
+// parseParams parses index class parameters defined in HCL or returned
+// from the database. For example: '{k=v}', '(k1=v1,k2=v2)'.
+func (o *IndexOpClass) parseParams(kv string) error {
+	switch {
+	case kv == "":
+	case strings.HasPrefix(kv, "(") && strings.HasSuffix(kv, ")"), strings.HasPrefix(kv, "{") && strings.HasSuffix(kv, "}"):
+		for _, e := range strings.Split(kv[1:len(kv)-1], ",") {
+			if kv := strings.Split(strings.TrimSpace(e), "="); len(kv) == 2 {
+				o.Params = append(o.Params, struct{ N, V string }{N: kv[0], V: kv[1]})
+			}
+		}
+	default:
+		return fmt.Errorf("postgres: unexpected operator class parameters format: %q", kv)
+	}
+	return nil
+}
+
 // newIndexStorage parses and returns the index storage parameters.
 func newIndexStorage(opts string) (*IndexStorageParams, error) {
 	params := &IndexStorageParams{}
@@ -992,35 +1144,44 @@ WHERE
 ORDER BY
 	t1.table_name, t1.ordinal_position
 `
+
 	fksQuery = `
-SELECT
-    t1.constraint_name,
-    t1.table_name,
-    t2.column_name,
-    t1.table_schema,
-    t3.table_name AS referenced_table_name,
-    t3.column_name AS referenced_column_name,
-    t3.table_schema AS referenced_schema_name,
-    t4.update_rule,
-    t4.delete_rule
-FROM
-    information_schema.table_constraints t1
-    JOIN information_schema.key_column_usage t2
-    ON t1.constraint_name = t2.constraint_name
-    AND t1.table_schema = t2.constraint_schema
-    JOIN information_schema.constraint_column_usage t3
-    ON t1.constraint_name = t3.constraint_name
-    AND t1.table_schema = t3.constraint_schema
-    JOIN information_schema.referential_constraints t4
-    ON t1.constraint_name = t4.constraint_name
-    AND t1.table_schema = t4.constraint_schema
-WHERE
-    t1.constraint_type = 'FOREIGN KEY'
-    AND t1.table_schema = $1
-    AND t1.table_name IN (%s)
-ORDER BY
-    t1.constraint_name,
-    t2.ordinal_position
+SELECT 
+    fk.constraint_name,
+    fk.table_name,
+    a1.attname AS column_name,
+    fk.schema_name,
+    fk.referenced_table_name,
+    a2.attname AS referenced_column_name,
+    fk.referenced_schema_name,
+    rc.update_rule,
+    rc.delete_rule
+	FROM 
+	    (
+	    	SELECT
+	      		con.conname AS constraint_name,
+	      		con.conrelid,
+	      		con.confrelid,
+	      		t1.relname AS table_name,
+	      		ns1.nspname AS schema_name,
+      			t2.relname AS referenced_table_name,
+	      		ns2.nspname AS referenced_schema_name,
+	      		unnest(con.conkey) AS conkey,
+	      		unnest(con.confkey) AS confkey
+	    	FROM pg_constraint con
+	    	JOIN pg_class t1 ON t1.oid = con.conrelid
+	    	JOIN pg_class t2 ON t2.oid = con.confrelid
+	    	JOIN pg_namespace ns1 on t1.relnamespace = ns1.oid
+	    	JOIN pg_namespace ns2 on t2.relnamespace = ns2.oid
+	    	WHERE ns1.nspname = $1
+	    	AND t1.relname IN (%s)
+	    	AND con.contype = 'f'
+	) AS fk
+	JOIN pg_attribute a1 ON a1.attnum = fk.conkey AND a1.attrelid = fk.conrelid
+	JOIN pg_attribute a2 ON a2.attnum = fk.confkey AND a2.attrelid = fk.confrelid
+	JOIN information_schema.referential_constraints rc ON rc.constraint_name = fk.constraint_name AND rc.constraint_schema = fk.schema_name
+	ORDER BY
+	    fk.conrelid, fk.constraint_name
 `
 
 	// Query to list table check constraints.
@@ -1062,14 +1223,17 @@ SELECT
 	%s AS included,
 	idx.indisprimary AS primary,
 	idx.indisunique AS unique,
-	c.contype AS constraint_type,
+	con.nametypes AS constraints,
 	pg_get_expr(idx.indpred, idx.indrelid) AS predicate,
 	pg_get_indexdef(idx.indexrelid, idx.ord, false) AS expression,
-	pg_index_column_has_property(idx.indexrelid, idx.ord, 'desc') AS desc,
+	pg_index_column_has_property(idx.indexrelid, idx.ord, 'desc') AS isdesc,
 	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_first') AS nulls_first,
 	pg_index_column_has_property(idx.indexrelid, idx.ord, 'nulls_last') AS nulls_last,
 	obj_description(i.oid, 'pg_class') AS comment,
-	i.reloptions AS options
+	i.reloptions AS options,
+	op.opcname AS opclass_name,
+	op.opcdefault AS opclass_default,
+	a2.attoptions AS opclass_params
 FROM
 	(
 		select
@@ -1081,13 +1245,18 @@ FROM
 	JOIN pg_class i ON i.oid = idx.indexrelid
 	JOIN pg_class t ON t.oid = idx.indrelid
 	JOIN pg_namespace n ON n.oid = t.relnamespace
-	LEFT JOIN pg_constraint c ON idx.indexrelid = c.conindid
+	LEFT JOIN (
+	    select conindid, jsonb_object_agg(conname, contype) AS nametypes
+	    from pg_constraint
+	    group by conindid
+	) con ON con.conindid = idx.indexrelid
 	LEFT JOIN pg_attribute a ON (a.attrelid, a.attnum) = (idx.indrelid, idx.key)
 	JOIN pg_am am ON am.oid = i.relam
+	LEFT JOIN pg_opclass op ON op.oid = idx.indclass[idx.ord-1]
+	LEFT JOIN pg_attribute a2 ON (a2.attrelid, a2.attnum) = (idx.indexrelid, idx.ord)
 WHERE
 	n.nspname = $1
 	AND t.relname IN (%s)
-	AND COALESCE(c.contype, '') <> 'f'
 ORDER BY
 	table_name, index_name, idx.ord
 `
