@@ -145,7 +145,7 @@ func TestPlanChanges(t *testing.T) {
 							&schema.DropIndex{
 								I: schema.NewUniqueIndex("unique_nickname").
 									AddColumns(schema.NewColumn("nickname")).
-									AddAttrs(&ConType{T: "u"}),
+									AddAttrs(&Constraint{T: "u"}),
 							},
 						},
 					}
@@ -395,6 +395,30 @@ func TestPlanChanges(t *testing.T) {
 									},
 								},
 							},
+							&schema.AddIndex{
+								I: &schema.Index{
+									Name: "concurrently",
+									Parts: []*schema.IndexPart{
+										{C: users.Columns[0]},
+									},
+									Attrs: []schema.Attr{
+										&Concurrently{},
+									},
+								},
+							},
+							&schema.AddIndex{
+								I: &schema.Index{
+									Name: "operator_class",
+									Parts: []*schema.IndexPart{
+										{C: users.Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "int8_bloom_ops"}}},
+										{C: users.Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "int8_minmax_ops"}}},
+										{C: users.Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "int8_minmax_multi_ops", Params: []struct{ N, V string }{{"values_per_range", "8"}}}}},
+									},
+									Attrs: []schema.Attr{
+										&IndexType{T: IndexTypeBRIN},
+									},
+								},
+							},
 						},
 					}
 				}(),
@@ -419,6 +443,14 @@ func TestPlanChanges(t *testing.T) {
 						Cmd:     `CREATE INDEX "include_key" ON "users" ("id") INCLUDE ("a", "b")`,
 						Reverse: `DROP INDEX "include_key"`,
 					},
+					{
+						Cmd:     `CREATE INDEX CONCURRENTLY "concurrently" ON "users" ("id")`,
+						Reverse: `DROP INDEX CONCURRENTLY "concurrently"`,
+					},
+					{
+						Cmd:     `CREATE INDEX "operator_class" ON "users" USING BRIN ("id" int8_bloom_ops, "id", "id" int8_minmax_multi_ops(values_per_range=8))`,
+						Reverse: `DROP INDEX "operator_class"`,
+					},
 					{
 						Cmd:     `COMMENT ON COLUMN "users" ."name" IS 'foo'`,
 						Reverse: `COMMENT ON COLUMN "users" ."name" IS ''`,
