@@ -13,6 +13,7 @@ import (
 	"ariga.io/atlas/schemahcl"
 	"ariga.io/atlas/sql/internal/specutil"
 	"ariga.io/atlas/sql/internal/sqlx"
+	"ariga.io/atlas/sql/postgres/internal/postgresop"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlspec"
 
@@ -111,12 +112,18 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 var (
 	hclState = schemahcl.New(
 		schemahcl.WithTypes(TypeRegistry.Specs()),
-		schemahcl.WithScopedEnums("table.index.type", IndexTypeBTree, IndexTypeHash, IndexTypeGIN, IndexTypeGiST, IndexTypeBRIN),
+		schemahcl.WithScopedEnums("table.index.type", IndexTypeBTree, IndexTypeBRIN, IndexTypeHash, IndexTypeGIN, IndexTypeGiST, "GiST", IndexTypeSPGiST, "SPGiST"),
 		schemahcl.WithScopedEnums("table.partition.type", PartitionTypeRange, PartitionTypeList, PartitionTypeHash),
 		schemahcl.WithScopedEnums("table.column.identity.generated", GeneratedTypeAlways, GeneratedTypeByDefault),
 		schemahcl.WithScopedEnums("table.column.as.type", "STORED"),
 		schemahcl.WithScopedEnums("table.foreign_key.on_update", specutil.ReferenceVars...),
 		schemahcl.WithScopedEnums("table.foreign_key.on_delete", specutil.ReferenceVars...),
+		schemahcl.WithScopedEnums("table.index.on.ops", func() (ops []string) {
+			for _, op := range postgresop.Classes {
+				ops = append(ops, op.Name)
+			}
+			return ops
+		}()...),
 	)
 	// MarshalHCL marshals v into an Atlas HCL DDL document.
 	MarshalHCL = schemahcl.MarshalerFunc(func(v any) ([]byte, error) {
@@ -208,27 +215,27 @@ func fromPartition(p Partition) *schemahcl.Resource {
 			specutil.VarAttr("type", strings.ToUpper(specutil.Var(p.T))),
 		},
 	}
-	columns, ok := func() (*schemahcl.ListValue, bool) {
-		parts := make([]schemahcl.Value, 0, len(p.Parts))
+	columns, ok := func() ([]*schemahcl.Ref, bool) {
+		parts := make([]*schemahcl.Ref, 0, len(p.Parts))
 		for _, p := range p.Parts {
 			if p.C == nil {
 				return nil, false
 			}
 			parts = append(parts, specutil.ColumnRef(p.C.Name))
 		}
-		return &schemahcl.ListValue{V: parts}, true
+		return parts, true
 	}()
 	if ok {
-		key.Attrs = append(key.Attrs, &schemahcl.Attr{K: "columns", V: columns})
+		key.Attrs = append(key.Attrs, schemahcl.RefsAttr("columns", columns...))
 		return key
 	}
 	for _, p := range p.Parts {
 		part := &schemahcl.Resource{Type: "by"}
 		switch {
 		case p.C != nil:
-			part.Attrs = append(part.Attrs, specutil.RefAttr("column", specutil.ColumnRef(p.C.Name)))
+			part.Attrs = append(part.Attrs, schemahcl.RefAttr("column", specutil.ColumnRef(p.C.Name)))
 		case p.X != nil:
-			part.Attrs = append(part.Attrs, specutil.StrAttr("expr", p.X.(*schema.RawExpr).X))
+			part.Attrs = append(part.Attrs, schemahcl.StringAttr("expr", p.X.(*schema.RawExpr).X))
 		}
 		key.Children = append(key.Children, part)
 	}
@@ -237,7 +244,7 @@ func fromPartition(p Partition) *schemahcl.Resource {
 
 // convertColumn converts a sqlspec.Column into a schema.Column.
 func convertColumn(spec *sqlspec.Column, _ *schema.Table) (*schema.Column, error) {
-	if err := fixDefaultQuotes(spec.Default); err != nil {
+	if err := fixDefaultQuotes(spec); err != nil {
 		return nil, err
 	}
 	c, err := specutil.Column(spec, convertColumnType)
@@ -278,24 +285,24 @@ func convertIdentity(r *schemahcl.Resource) (*Identity, error) {
 
 // fixDefaultQuotes fixes the quotes on the Default field to be single quotes
 // instead of double quotes.
-func fixDefaultQuotes(value schemahcl.Value) error {
-	lv, ok := value.(*schemahcl.LiteralValue)
-	if !ok {
+func fixDefaultQuotes(spec *sqlspec.Column) error {
+	if spec.Default.Type() != cty.String {
 		return nil
 	}
-	if sqlx.IsQuoted(lv.V, '"') {
-		uq, err := strconv.Unquote(lv.V)
+	if s := spec.Default.AsString(); sqlx.IsQuoted(s, '"') {
+		uq, err := strconv.Unquote(s)
 		if err != nil {
 			return err
 		}
-		lv.V = "'" + uq + "'"
+		s = "'" + uq + "'"
+		spec.Default = cty.StringVal(s)
 	}
 	return nil
 }
 
 // convertIndex converts a sqlspec.Index into a schema.Index.
 func convertIndex(spec *sqlspec.Index, t *schema.Table) (*schema.Index, error) {
-	idx, err := specutil.Index(spec, t)
+	idx, err := specutil.Index(spec, t, convertPart)
 	if err != nil {
 		return nil, err
 	}
@@ -304,7 +311,7 @@ func convertIndex(spec *sqlspec.Index, t *schema.Table) (*schema.Index, error) {
 		if err != nil {
 			return nil, err
 		}
-		idx.Attrs = append(idx.Attrs, &IndexType{T: t})
+		idx.Attrs = append(idx.Attrs, &IndexType{T: strings.ToUpper(t)})
 	}
 	if attr, ok := spec.Attr("where"); ok {
 		p, err := attr.String()
@@ -339,6 +346,37 @@ func convertIndex(spec *sqlspec.Index, t *schema.Table) (*schema.Index, error) {
 	return idx, nil
 }
 
+func convertPart(spec *sqlspec.IndexPart, part *schema.IndexPart) error {
+	switch opc, ok := spec.Attr("ops"); {
+	case !ok:
+	case opc.IsRawExpr():
+		expr, err := opc.RawExpr()
+		if err != nil {
+			return err
+		}
+		var op IndexOpClass
+		if err := op.UnmarshalText([]byte(expr.X)); err != nil {
+			return fmt.Errorf("unexpected index.on.ops expression %q: %w", expr.X, err)
+		}
+		if op.Name != "" {
+			part.Attrs = append(part.Attrs, &op)
+		}
+	case opc.IsRef():
+		name, err := opc.Ref()
+		if err != nil {
+			return err
+		}
+		part.Attrs = append(part.Attrs, &IndexOpClass{Name: name})
+	default:
+		name, err := opc.String()
+		if err != nil {
+			return err
+		}
+		part.Attrs = append(part.Attrs, &IndexOpClass{Name: name})
+	}
+	return nil
+}
+
 const defaultTimePrecision = 6
 
 // convertColumnType converts a sqlspec.Column into a concrete Postgres schema.Type.
@@ -492,33 +530,47 @@ func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
 }
 
 func indexSpec(idx *schema.Index) (*sqlspec.Index, error) {
-	spec, err := specutil.FromIndex(idx)
+	spec, err := specutil.FromIndex(idx, partAttr)
 	if err != nil {
 		return nil, err
 	}
 	// Avoid printing the index type if it is the default.
-	if i := (IndexType{}); sqlx.Has(idx.Attrs, &i) && i.T != IndexTypeBTree {
+	if i := (IndexType{}); sqlx.Has(idx.Attrs, &i) && strings.ToUpper(i.T) != IndexTypeBTree {
 		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("type", strings.ToUpper(i.T)))
 	}
 	if i := (IndexInclude{}); sqlx.Has(idx.Attrs, &i) && len(i.Columns) > 0 {
-		attr := &schemahcl.ListValue{}
+		refs := make([]*schemahcl.Ref, 0, len(i.Columns))
 		for _, c := range i.Columns {
-			attr.V = append(attr.V, specutil.ColumnRef(c.Name))
+			refs = append(refs, specutil.ColumnRef(c.Name))
 		}
-		spec.Extra.Attrs = append(spec.Extra.Attrs, &schemahcl.Attr{
-			K: "include",
-			V: attr,
-		})
+		spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.RefsAttr("include", refs...))
 	}
 	if i := (IndexPredicate{}); sqlx.Has(idx.Attrs, &i) && i.P != "" {
 		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("where", strconv.Quote(i.P)))
 	}
 	if p, ok := indexStorageParams(idx.Attrs); ok {
-		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.Int64Attr("page_per_range", p.PagesPerRange))
+		spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.Int64Attr("page_per_range", p.PagesPerRange))
 	}
 	return spec, nil
 }
 
+func partAttr(idx *schema.Index, part *schema.IndexPart, spec *sqlspec.IndexPart) error {
+	var op IndexOpClass
+	if !sqlx.Has(part.Attrs, &op) {
+		return nil
+	}
+	switch d, err := op.DefaultFor(idx, part); {
+	case err != nil:
+		return err
+	case d:
+	case len(op.Params) > 0:
+		spec.Extra.Attrs = append(spec.Extra.Attrs, schemahcl.RawAttr("ops", op.String()))
+	default:
+		spec.Extra.Attrs = append(spec.Extra.Attrs, specutil.VarAttr("ops", op.String()))
+	}
+	return nil
+}
+
 // columnSpec converts from a concrete Postgres schema.Column into a sqlspec.Column.
 func columnSpec(c *schema.Column, _ *schema.Table) (*sqlspec.Column, error) {
 	s, err := specutil.FromColumn(c, columnTypeSpec)
@@ -544,10 +596,10 @@ func fromIdentity(i *Identity) *schemahcl.Resource {
 	}
 	if s := i.Sequence; s != nil {
 		if s.Start != 1 {
-			id.Attrs = append(id.Attrs, specutil.Int64Attr("start", s.Start))
+			id.Attrs = append(id.Attrs, schemahcl.Int64Attr("start", s.Start))
 		}
 		if s.Increment != 1 {
-			id.Attrs = append(id.Attrs, specutil.Int64Attr("increment", s.Increment))
+			id.Attrs = append(id.Attrs, schemahcl.Int64Attr("increment", s.Increment))
 		}
 	}
 	return id
@@ -625,6 +677,8 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.NewTypeSpec(TypeJSONB),
 		schemahcl.NewTypeSpec(TypeUUID),
 		schemahcl.NewTypeSpec(TypeMoney),
+		schemahcl.NewTypeSpec(TypeTSVector),
+		schemahcl.NewTypeSpec(TypeTSQuery),
 		schemahcl.NewTypeSpec("hstore"),
 		schemahcl.NewTypeSpec("sql", schemahcl.WithAttributes(&schemahcl.TypeAttr{Name: "def", Required: true, Kind: reflect.String})),
 	),
@@ -640,7 +694,7 @@ var TypeRegistry = schemahcl.NewRegistry(
 					spec.T = specutil.Var(strings.ToLower(i.F))
 				}
 				if p := i.Precision; p != nil && *p != defaultTimePrecision {
-					spec.Attrs = []*schemahcl.Attr{specutil.IntAttr("precision", *p)}
+					spec.Attrs = []*schemahcl.Attr{schemahcl.IntAttr("precision", *p)}
 				}
 				return spec, nil
 			}),
@@ -692,7 +746,7 @@ func typeSpec(t schema.Type) (*schemahcl.Type, error) {
 	if t, ok := t.(*schema.TimeType); ok && t.T != TypeDate {
 		spec := &schemahcl.Type{T: timeAlias(t.T)}
 		if p := t.Precision; p != nil && *p != defaultTimePrecision {
-			spec.Attrs = []*schemahcl.Attr{specutil.IntAttr("precision", *p)}
+			spec.Attrs = []*schemahcl.Attr{schemahcl.IntAttr("precision", *p)}
 		}
 		return spec, nil
 	}
