@@ -10,6 +10,7 @@ import (
 	"testing"
 
 	"ariga.io/atlas/sql/internal/spectest"
+	"ariga.io/atlas/sql/internal/sqlx"
 	"ariga.io/atlas/sql/schema"
 	"github.com/stretchr/testify/require"
 )
@@ -283,12 +284,17 @@ table "t" {
 	t.Run("Valid", func(t *testing.T) {
 		var (
 			s schema.Schema
-			f = fmt.Sprintf(f, "HASH")
+			r = fmt.Sprintf(f, "HASH")
 		)
-		err := EvalHCLBytes([]byte(f), &s, nil)
-		require.NoError(t, err)
+		require.NoError(t, EvalHCLBytes([]byte(r), &s, nil))
 		idx := s.Tables[0].Indexes[0]
 		require.Equal(t, IndexTypeHash, idx.Attrs[0].(*IndexType).T)
+
+		s = schema.Schema{}
+		r = fmt.Sprintf(f, "GiST")
+		require.NoError(t, EvalHCLBytes([]byte(r), &s, nil))
+		idx = s.Tables[0].Indexes[0]
+		require.Equal(t, IndexTypeGiST, idx.Attrs[0].(*IndexType).T)
 	})
 }
 
@@ -315,6 +321,95 @@ table "t" {
 	require.EqualValues(t, 2, idx.Attrs[1].(*IndexStorageParams).PagesPerRange)
 }
 
+func TestUnmarshalSpec_IndexOpClass(t *testing.T) {
+	const f = `table "users" {
+  schema = schema.test
+  column "a" {
+    null = false
+    type = text
+  }
+  column "b" {
+    null = false
+    type = tsvector
+  }
+  index "idx0" {
+    unique  = true
+    columns = [column.a, column.b]
+  }
+  index "idx1" {
+    unique = true
+    on {
+      column = column.a
+      ops    = text_pattern_ops
+    }
+    on {
+      column = column.b
+    }
+  }
+  index "idx2" {
+    unique  = true
+    columns = [column.a]
+  }
+  index "idx3" {
+    unique = true
+    on {
+      column = column.a
+      ops    = text_pattern_ops
+    }
+  }
+  index "idx4" {
+    unique = true
+    type   = GIST
+    on {
+      column = column.b
+      ops    = sql("tsvector_ops(siglen=1)")
+    }
+  }
+}
+schema "test" {
+}
+`
+	var s schema.Schema
+	err := EvalHCLBytes([]byte(f), &s, nil)
+	require.NoError(t, err)
+	// idx0
+	idx := s.Tables[0].Indexes[0]
+	require.Len(t, idx.Parts, 2)
+	require.Equal(t, "a", idx.Parts[0].C.Name)
+	require.Empty(t, idx.Parts[0].Attrs)
+	require.Equal(t, "b", idx.Parts[1].C.Name)
+	require.Empty(t, idx.Parts[1].Attrs)
+	// idx1
+	idx = s.Tables[0].Indexes[1]
+	require.Len(t, idx.Parts, 2)
+	require.Equal(t, "a", idx.Parts[0].C.Name)
+	require.Len(t, idx.Parts[0].Attrs, 1)
+	require.Equal(t, "text_pattern_ops", idx.Parts[0].Attrs[0].(*IndexOpClass).Name)
+	require.Equal(t, "b", idx.Parts[1].C.Name)
+	require.Empty(t, idx.Parts[1].Attrs)
+	// idx2
+	idx = s.Tables[0].Indexes[2]
+	require.Len(t, idx.Parts, 1)
+	require.Equal(t, "a", idx.Parts[0].C.Name)
+	require.Empty(t, idx.Parts[0].Attrs)
+	// idx3
+	idx = s.Tables[0].Indexes[3]
+	require.Len(t, idx.Parts, 1)
+	require.Equal(t, "a", idx.Parts[0].C.Name)
+	require.Len(t, idx.Parts[0].Attrs, 1)
+	require.Equal(t, "text_pattern_ops", idx.Parts[0].Attrs[0].(*IndexOpClass).Name)
+	// idx4
+	idx = s.Tables[0].Indexes[4]
+	require.Len(t, idx.Parts, 1)
+	require.Equal(t, "b", idx.Parts[0].C.Name)
+	require.Len(t, idx.Parts[0].Attrs, 1)
+	opc := idx.Parts[0].Attrs[0].(*IndexOpClass)
+	require.Equal(t, "tsvector_ops", opc.Name)
+	require.Len(t, opc.Params, 1)
+	require.Equal(t, "siglen", opc.Params[0].N)
+	require.Equal(t, "1", opc.Params[0].V)
+}
+
 func TestUnmarshalSpec_Partitioned(t *testing.T) {
 	t.Run("Columns", func(t *testing.T) {
 		var (
@@ -569,6 +664,138 @@ schema "test" {
 	require.EqualValues(t, expected, string(buf))
 }
 
+func TestMarshalSpec_IndexOpClass(t *testing.T) {
+	s := &schema.Schema{
+		Name: "test",
+		Tables: []*schema.Table{
+			{
+				Name: "users",
+				Columns: []*schema.Column{
+					{
+						Name: "a",
+						Type: &schema.ColumnType{Type: &schema.StringType{T: "text"}},
+					},
+					{
+						Name: "b",
+						Type: &schema.ColumnType{Type: &TextSearchType{T: "tsvector"}},
+					},
+				},
+			},
+		},
+	}
+	s.Tables[0].Schema = s
+	s.Tables[0].Schema = s
+	s.Tables[0].Indexes = []*schema.Index{
+		{
+			Name:   "idx0",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "text_ops"}}},
+				{SeqNo: 1, C: s.Tables[0].Columns[1], Attrs: []schema.Attr{&IndexOpClass{Name: "tsvector_ops"}}},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: IndexTypeBTree},
+			},
+		},
+		{
+			Name:   "idx1",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "text_pattern_ops"}}},
+				{SeqNo: 1, C: s.Tables[0].Columns[1], Attrs: []schema.Attr{&IndexOpClass{Name: "tsvector_ops"}}},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: IndexTypeBTree},
+			},
+		},
+		{
+			Name:   "idx2",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "text_ops"}}},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: IndexTypeBTree},
+			},
+		},
+		{
+			Name:   "idx3",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "text_pattern_ops"}}},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: IndexTypeBTree},
+			},
+		},
+		{
+			Name:   "idx4",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[1], Attrs: []schema.Attr{&IndexOpClass{Name: "tsvector_ops", Params: []struct{ N, V string }{{"siglen", "1"}}}}},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: IndexTypeGiST},
+			},
+		},
+	}
+	buf, err := MarshalSpec(s, hclState)
+	require.NoError(t, err)
+	const expected = `table "users" {
+  schema = schema.test
+  column "a" {
+    null = false
+    type = text
+  }
+  column "b" {
+    null = false
+    type = tsvector
+  }
+  index "idx0" {
+    unique  = true
+    columns = [column.a, column.b]
+  }
+  index "idx1" {
+    unique = true
+    on {
+      column = column.a
+      ops    = text_pattern_ops
+    }
+    on {
+      column = column.b
+    }
+  }
+  index "idx2" {
+    unique  = true
+    columns = [column.a]
+  }
+  index "idx3" {
+    unique = true
+    on {
+      column = column.a
+      ops    = text_pattern_ops
+    }
+  }
+  index "idx4" {
+    unique = true
+    type   = GIST
+    on {
+      column = column.b
+      ops    = sql("tsvector_ops(siglen=1)")
+    }
+  }
+}
+schema "test" {
+}
+`
+	require.EqualValues(t, expected, string(buf))
+}
+
 func TestUnmarshalSpec_Identity(t *testing.T) {
 	f := `
 schema "s" {}
@@ -602,6 +829,99 @@ table "t" {
 	})
 }
 
+func TestUnmarshalSpec_IndexInclude(t *testing.T) {
+	f := `
+schema "s" {}
+table "t" {
+	schema = schema.s
+	column "c" {
+		type = int
+	}
+	column "d" {
+		type = int
+	}
+	index "c" {
+		columns = [
+			column.c,
+		]
+		include = [
+			column.d,
+		]
+	}
+}
+`
+	var s schema.Schema
+	err := EvalHCLBytes([]byte(f), &s, nil)
+	require.NoError(t, err)
+	require.Len(t, s.Tables, 1)
+	require.Len(t, s.Tables[0].Columns, 2)
+	require.Len(t, s.Tables[0].Indexes, 1)
+	idx, ok := s.Tables[0].Index("c")
+	require.True(t, ok)
+	require.Len(t, idx.Parts, 1)
+	require.Len(t, idx.Attrs, 1)
+	var include IndexInclude
+	require.True(t, sqlx.Has(idx.Attrs, &include))
+	require.Len(t, include.Columns, 1)
+	require.Equal(t, "d", include.Columns[0].Name)
+}
+
+func TestMarshalSpec_IndexInclude(t *testing.T) {
+	s := &schema.Schema{
+		Name: "test",
+		Tables: []*schema.Table{
+			{
+				Name: "users",
+				Columns: []*schema.Column{
+					{
+						Name: "c",
+						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "int"}},
+					},
+					{
+						Name: "d",
+						Type: &schema.ColumnType{Type: &schema.IntegerType{T: "int"}},
+					},
+				},
+			},
+		},
+	}
+	s.Tables[0].Schema = s
+	s.Tables[0].Schema = s
+	s.Tables[0].Indexes = []*schema.Index{
+		{
+			Name:  "index",
+			Table: s.Tables[0],
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0]},
+			},
+			Attrs: []schema.Attr{
+				&IndexInclude{Columns: s.Tables[0].Columns[1:]},
+			},
+		},
+	}
+	buf, err := MarshalSpec(s, hclState)
+	require.NoError(t, err)
+	const expected = `table "users" {
+  schema = schema.test
+  column "c" {
+    null = false
+    type = int
+  }
+  column "d" {
+    null = false
+    type = int
+  }
+  index "index" {
+    columns = [column.c]
+    include = [column.d]
+  }
+}
+schema "test" {
+}
+`
+	require.EqualValues(t, expected, string(buf))
+}
+
 func TestMarshalSpec_GeneratedColumn(t *testing.T) {
 	s := schema.New("test").
 		AddTables(
