@@ -48,7 +48,7 @@ func init() {
 	sqlclient.Register(
 		DriverName,
 		sqlclient.DriverOpener(Open),
-		sqlclient.RegisterTxOpener(openTx),
+		sqlclient.RegisterTxOpener(OpenTx),
 		sqlclient.RegisterCodec(MarshalHCL, EvalHCL),
 		sqlclient.RegisterFlavours("sqlite"),
 		sqlclient.RegisterURLParser(sqlclient.URLParserFunc(func(u *url.URL) *sqlclient.URL {
@@ -73,17 +73,17 @@ func Open(db schema.ExecQuerier) (migrate.Driver, error) {
 		return nil, fmt.Errorf("sqlite: query version pragma: %w", err)
 	}
 	if err := sqlx.ScanOne(rows, &c.version); err != nil {
-		return nil, fmt.Errorf("sqlite: scan version and foreign_keys pragma: %w", err)
+		return nil, fmt.Errorf("sqlite: scan version pragma: %w", err)
 	}
 	if rows, err = db.QueryContext(ctx, "SELECT name FROM pragma_collation_list()"); err != nil {
-		return nil, fmt.Errorf("sqlite: query foreign_keys pragma: %w", err)
+		return nil, fmt.Errorf("sqlite: query collation_list pragma: %w", err)
 	}
 	if c.collations, err = sqlx.ScanStrings(rows); err != nil {
 		return nil, fmt.Errorf("sqlite: scanning database collations: %w", err)
 	}
 	return &Driver{
 		conn:        c,
-		Differ:      &sqlx.Diff{DiffDriver: &Diff{}},
+		Differ:      &sqlx.Diff{DiffDriver: &diff{}},
 		Inspector:   &inspect{c},
 		PlanApplier: &planApply{c},
 	}, nil
@@ -165,7 +165,15 @@ func acquireLock(path string, timeout time.Duration) (schema.UnlockFunc, error)
 	return func() error { return os.Remove(path) }, nil
 }
 
-func openTx(ctx context.Context, db *sql.DB, opts *sql.TxOptions) (*sqlclient.Tx, error) {
+type violation struct {
+	tbl, ref   string
+	row, index int
+}
+
+// OpenTx opens a transaction. If foreign keys are enabled, it disables them, checks for constraint violations,
+// opens the transaction and before committing ensures no new violations have been introduced by whatever Atlas was
+// doing.
+func OpenTx(ctx context.Context, db *sql.DB, opts *sql.TxOptions) (*sqlclient.Tx, error) {
 	var on sql.NullBool
 	if err := db.QueryRowContext(ctx, "PRAGMA foreign_keys").Scan(&on); err != nil {
 		return nil, fmt.Errorf("sql/sqlite: querying 'foreign_keys' pragma: %w", err)
@@ -173,7 +181,8 @@ func openTx(ctx context.Context, db *sql.DB, opts *sql.TxOptions) (*sqlclient.Tx
 	// Disable the foreign_keys pragma in case it is enabled, and
 	// toggle it back after transaction is committed or rolled back.
 	if on.Bool {
-		if _, err := db.ExecContext(ctx, "PRAGMA foreign_keys = off"); err != nil {
+		_, err := db.ExecContext(ctx, "PRAGMA foreign_keys = off")
+		if err != nil {
 			return nil, fmt.Errorf("sql/sqlite: set 'foreign_keys = off': %w", err)
 		}
 	}
@@ -181,19 +190,118 @@ func openTx(ctx context.Context, db *sql.DB, opts *sql.TxOptions) (*sqlclient.Tx
 	if err != nil {
 		return nil, err
 	}
+	cm, err := CommitFunc(ctx, db, tx, on.Bool)
+	if err != nil {
+		return nil, err
+	}
 	return &sqlclient.Tx{
-		Tx: tx,
-		Close: func() error {
-			if on.Bool {
-				if _, err := db.ExecContext(ctx, "PRAGMA foreign_keys = on"); err != nil {
-					return fmt.Errorf("sql/sqlite: set 'foreign_keys = on': %w", err)
-				}
-			}
-			return nil
-		},
+		Tx:         tx,
+		CommitFn:   cm,
+		RollbackFn: RollbackFunc(ctx, db, tx, on.Bool),
 	}, nil
 }
 
+// Tx wraps schema.ExecQuerier with the transaction methods.
+type Tx interface {
+	schema.ExecQuerier
+	Commit() error
+	Rollback() error
+}
+
+// CommitFunc takes a transaction and ensures to toggle foreign keys back on after tx.Commit is called.
+func CommitFunc(ctx context.Context, db schema.ExecQuerier, tx Tx, on bool) (func() error, error) {
+	var (
+		before []violation
+		err    error
+	)
+	if on {
+		before, err = violations(ctx, tx)
+		if err != nil {
+			return nil, err
+		}
+	}
+	return func() error {
+		if on {
+			after, err := violations(ctx, tx)
+			if err != nil {
+				if err2 := tx.Rollback(); err2 != nil {
+					err = fmt.Errorf("%v: %w", err2, err)
+				}
+				return enableFK(ctx, db, on, err)
+			}
+			if vs := violationsDiff(before, after); len(vs) > 0 {
+				err := fmt.Errorf("sql/sqlite: foreign key mismatch: %+v", vs)
+				if err2 := tx.Rollback(); err2 != nil {
+					err = fmt.Errorf("%v: %w", err2, err)
+				}
+				return enableFK(ctx, db, on, err)
+			}
+		}
+		return enableFK(ctx, db, on, tx.Commit())
+	}, nil
+}
+
+// RollbackFunc takes a transaction and ensures to toggle foreign keys back on after tx.Rollback is called.
+func RollbackFunc(ctx context.Context, db schema.ExecQuerier, tx Tx, on bool) func() error {
+	return func() error {
+		return enableFK(ctx, db, on, tx.Rollback())
+	}
+}
+
+func enableFK(ctx context.Context, db schema.ExecQuerier, do bool, err error) error {
+	if do {
+		// Re-enable foreign key checks if they were enabled before.
+		if _, err2 := db.ExecContext(ctx, "PRAGMA foreign_keys = on"); err2 != nil {
+			err2 = fmt.Errorf("sql/sqlite: set 'foreign_keys = on': %w", err2)
+			if err != nil {
+				return fmt.Errorf("%v: %w", err2, err)
+			}
+			return err2
+		}
+	}
+	return err
+}
+
+func violations(ctx context.Context, conn schema.ExecQuerier) ([]violation, error) {
+	rows, err := conn.QueryContext(ctx, "PRAGMA foreign_key_check")
+	if err != nil {
+		return nil, fmt.Errorf("sql/sqlite: querying 'foreign_key_check' pragma: %w", err)
+	}
+	defer rows.Close()
+	var vs []violation
+	for rows.Next() {
+		var v violation
+		if err := rows.Scan(&v.tbl, &v.row, &v.ref, &v.index); err != nil {
+			return nil, fmt.Errorf("sql/sqlite: querying 'foreign_key_check' pragma: scanning rows: %w", err)
+		}
+		vs = append(vs, v)
+	}
+	if err := rows.Err(); err != nil {
+		return nil, fmt.Errorf("sql/sqlite: querying 'foreign_key_check' pragma: scanning rows: %w", err)
+	}
+	return vs, nil
+}
+
+// equalViolations compares the foreign key violations before starting a transaction with the ones afterwards.
+// It returns violations found in v2 that are not in v1.
+func violationsDiff(v1, v2 []violation) (vs []violation) {
+	for _, v := range v2 {
+		if !contains(v1, v) {
+			vs = append(vs, v)
+		}
+	}
+	return vs
+}
+
+func contains(hs []violation, n violation) bool {
+	for _, v := range hs {
+		if v.row == n.row && v.ref == n.ref && v.index == n.index && v.tbl == n.tbl {
+			return true
+		}
+	}
+	return false
+}
+
 // SQLite standard data types as defined in its codebase and documentation.
 // https://www.sqlite.org/datatype3.html
 // https://github.com/sqlite/sqlite/blob/master/src/global.c
